import json
from enum import Enum
from typing import List

none = "d3043820717d74d9a17694c176d39733"


# region Aks
class Aks:
    """
    # Arguments
    cluster_name: str
    infrastructure_resource_group_name: str
    region: str
    resource_group_name: str
    """

    def __init__(
            self,
            cluster_name: str = none,
            infrastructure_resource_group_name: str = none,
            region: str = none,
            resource_group_name: str = none):
        self.cluster_name = cluster_name
        self.infrastructure_resource_group_name = infrastructure_resource_group_name
        self.region = region
        self.resource_group_name = resource_group_name
# endregion

# region AutoScaler


class ResourceLimits:
    """
    # Arguments
    max_memory_gib: int
    max_v_cpu: int
    """

    def __init__(
            self,
            max_memory_gib: int = none,
            max_v_cpu: int = none):
        self.max_memory_gib = max_memory_gib
        self.max_v_cpu = max_v_cpu


class Down:
    """
    # Arguments
    max_scale_down_percentage: float
    """

    def __init__(
            self,
            max_scale_down_percentage: float = none):
        self.max_scale_down_percentage = max_scale_down_percentage


class Automatic:
    """
    # Arguments
    percentage: int
    """

    def __init__(
            self,
            percentage: int = none):
        self.percentage = percentage


class AutoScalerHeadroom:
    """
    # Arguments
    automatic: Automatic
    """

    def __init__(
            self,
            automatic: Automatic = none):
        self.automatic = automatic


class AutoScaler:
    """
    # Arguments
    down: Down
    headroom: AutoScalerHeadroom
    is_enabled: bool
    resource_limits: ResourceLimits
    """

    def __init__(
            self,
            down: Down = none,
            headroom: AutoScalerHeadroom = none,
            is_enabled: bool = none,
            resource_limits: ResourceLimits = none):
        self.down = down
        self.headroom = headroom
        self.is_enabled = is_enabled
        self.resource_limits = resource_limits
# endregion

# region Scheduling


class ShutdownHours:
    """
    # Arguments
    is_enabled: bool
    time_windows: List[str]
    """

    def __init__(
            self,
            is_enabled: bool = none,
            time_windows: List[str] = none):
        self.is_enabled = is_enabled
        self.time_windows = time_windows

class SuspensionHours:
    """
    # Arguments
    is_enabled: bool
    time_windows: List[str]
    """

    def __init__(
            self,
            is_enabled: bool = none,
            time_windows: List[str] = none):
        self.is_enabled = is_enabled
        self.time_windows = time_windows        


class ClusterRoll:
    """
    # Arguments
    batch_min_healthy_percentage: int
    batch_size_percentage: int
    comment: str
    respect_pdb: bool
    respect_restrict_scale_down: bool
    vng_ids: List[str]
    """

    def __init__(
            self,
            batch_min_healthy_percentage: int = none,
            batch_size_percentage: int = none,
            comment: str = none,
            respect_pdb: bool = none,
            respect_restrict_scale_down: bool = none,
            vng_ids: List[str] = none):
        self.batch_min_healthy_percentage = batch_min_healthy_percentage
        self.batch_size_percentage = batch_size_percentage
        self.comment = comment
        self.respect_pdb = respect_pdb
        self.respect_restrict_scale_down = respect_restrict_scale_down
        self.vng_ids = vng_ids


class Parameters:
    """
    # Arguments
    cluster_roll: ClusterRoll
    """

    def __init__(
            self,
            cluster_roll: ClusterRoll = none):
        self.cluster_roll = cluster_roll


class TaskType(Enum):
    cluster_roll = "clusterRoll"


class Task:
    """
    # Arguments
    cron_expression: str
    is_enabled: bool
    parameters: Parameters
    task_type: TaskType
    """

    def __init__(
            self,
            cron_expression: str = none,
            is_enabled: bool = none,
            parameters: Parameters = none,
            task_type: TaskType = none):
        self.cron_expression = cron_expression
        self.is_enabled = is_enabled
        self.parameters = parameters
        self.task_type = task_type


class Scheduling:
    """
    # Arguments
    shutdown_hours: ShutdownHours
    tasks: List[Task]
    """

    def __init__(
            self,
            shutdown_hours: ShutdownHours = none,
            suspension_hours : SuspensionHours = none,
            tasks: List[Task] = none):
        self.shutdown_hours = shutdown_hours
        self.suspension_hours = suspension_hours
        self.tasks = tasks
# endregion

# region Health


class Health:
    """
    # Arguments
    grace_period: int
    """

    def __init__(
            self,
            grace_period: int = none):
        self.grace_period = grace_period
 # endregion

# region VirtualNodeGroupTemplate

# region AutoScale


class AutoScaleHeadroom:
    """
    # Arguments
    cpu_per_unit: int
    gpu_per_unit: int
    memory_per_unit: int
    num_of_units: int
    """

    def __init__(
            self,
            cpu_per_unit: int = none,
            gpu_per_unit: int = none,
            memory_per_unit: int = none,
            num_of_units: int = none):
        self.cpu_per_unit = cpu_per_unit
        self.gpu_per_unit = gpu_per_unit
        self.memory_per_unit = memory_per_unit
        self.num_of_units = num_of_units


class AutoScale:
    """
    # Arguments
    headrooms: List[AutoScaleHeadroom]
    """

    def __init__(
            self,
            headrooms: List[AutoScaleHeadroom] = none):
        self.headrooms = headrooms
# endregion

# region NodeCountLimits


class NodeCountLimits:
    """
    # Arguments
    max_count: int
    min_count: int
    """

    def __init__(
            self,
            max_count: int = none,
            min_count: int = none):
        self.max_count = max_count
        self.min_count = min_count
# endregion

# region NodePoolProperties


class OsType(Enum):
    linux = "Linux"
    windows = "Windows"


class OsDiskType(Enum):
    managed = "Managed"
    ephemereal = "Ephemereal"


class OsSKU(Enum):
    ubuntu = "Ubuntu"
    azure_linux = "AzureLinux"
    cbl_mariner = "CBLMariner"
    windows2019 = "Windows2019"
    windows2022 = "Windows2022"


class Sysctls:
    """
    # Arguments
    vm_max_map_count: int
    """

    def __init__(
            self,
            vm_max_map_count: int = none):
        self.vm_max_map_count = vm_max_map_count


class LinuxOSConfig:
    """
    # Arguments
    sysctls: Sysctls
    """

    def __init__(
            self,
            sysctls: Sysctls = none):
        self.sysctls = sysctls


class NodePoolProperties:
    """
    # Arguments
    enable_node_public_i_p: bool
    kubernetes_version: str
    linux_o_s_config: LinuxOSConfig
    max_pods_per_node: int
    os_disk_size_g_b: int
    os_disk_type: OsDiskType
    os_s_k_u: OsSKU
    os_type: OsType
    pod_subnet_i_ds: List[str]
    vnet_subnet_i_ds: List[str]
    """

    def __init__(
            self,
            enable_node_public_i_p: bool = none,
            kubernetes_version: str = none,
            linux_o_s_config: LinuxOSConfig = none,
            max_pods_per_node: int = none,
            os_disk_size_g_b: int = none,
            os_disk_type: OsDiskType = none,
            os_s_k_u: OsSKU = none,
            os_type: OsType = none,
            pod_subnet_i_ds: List[str] = none,
            vnet_subnet_i_ds: List[str] = none):
        self.enable_node_public_i_p = enable_node_public_i_p
        self.kubernetes_version = kubernetes_version
        self.linux_o_s_config = linux_o_s_config
        self.max_pods_per_node = max_pods_per_node
        self.os_disk_size_g_b = os_disk_size_g_b
        self.os_disk_type = os_disk_type
        self.os_s_k_u = os_s_k_u
        self.os_type = os_type
        self.pod_subnet_i_ds = pod_subnet_i_ds
        self.vnet_subnet_i_ds = vnet_subnet_i_ds
# endregion

# region Strategy


class Strategy:
    """
    # Arguments
    fallback_to_od: bool
    spot_percentage: int
    """

    def __init__(
            self,
            fallback_to_od: bool = none,
            spot_percentage: int = none):
        self.fallback_to_od = fallback_to_od
        self.spot_percentage = spot_percentage
# endregion

# region Taint


class Effect(Enum):
    no_schedule = "NoSchedule"
    prefer_no_schedule = "PreferNoSchedule"
    no_execute = "NoExecute"
    prefer_no_execute = "PreferNoExecute"


class Taint:
    """
    # Arguments
    key: str
    value: str
    effect: Effect
    """

    def __init__(
            self,
            key: str = none,
            value: str = none,
            effect: Effect = none):
        self.key = key
        self.value = value
        self.effect = effect
# endregion

# region VmSizes


class Architecture(Enum):
    x86_64 = "X86_64"
    intel64 = "INTEL64"
    amd64 = "AMD64"
    arm64 = "ARM64"


class AcceleratedNetworking(Enum):
    enabled = "Enabled"
    disabled = "Disabled"


class DiskPerformance(Enum):
    standard = "Standard"
    premium = "Premium"


class VmType(Enum):
    general_purpose = "generalPurpose"
    memory_optimized = "memoryOptimized"
    compute_optimized = "computeOptimized"
    high_performance_compute = "highPerformanceCompute"
    storage_optimized = "storageOptimized"
    gpu = "GPU"


class GpuType(Enum):
    nvidia_tesla_v100 = "nvidia-tesla-v100"
    amd_radeon_instinct_mi25 = "amd-radeon-instinct-mi25"
    nvidia_a10 = "nvidia-a10"
    nvidia_tesla_a100 = "nvidia-tesla-a100"
    nvidia_tesla_k80 = "nvidia-tesla-k80"
    nvidia_tesla_m60 = "nvidia-tesla-m60"
    nvidia_tesla_p100 = "nvidia-tesla-p100"
    nvidia_tesla_p40 = "nvidia-tesla-p40"
    nvidia_tesla_t4 = "nvidia-tesla-t4"
    nvidia_tesla_h100 = "nvidia-tesla-h100"


class Filters:
    """
    # Arguments
    accelerated_networking: AcceleratedNetworking
    disk_performance: DiskPerformance
    architectures: List[Architecture]
    exclude_series: List[str]
    max_memory_gi_b: float
    max_v_cpu: int
    min_memory_gi_b: float
    min_v_cpu: int
    min_gpu: float
    max_gpu: float
    min_data: int
    min_n_i_cs: int
    series: List[str]
    vm_types: List[VmType]
    gpu_types: List[GpuType]
    """

    def __init__(
            self,
            accelerated_networking: AcceleratedNetworking = none,
            disk_performance: DiskPerformance = none,
            architectures: List[Architecture] = none,
            exclude_series: List[str] = none,
            max_memory_gi_b: float = none,
            max_v_cpu: int = none,
            min_memory_gi_b: float = none,
            min_v_cpu: int = none,
            min_gpu: float = none,
            max_gpu: float = none,
            min_data: int = none,
            min_n_i_cs: int = none,
            series: List[str] = none,
            vm_types: List[VmType] = none,
            gpu_types: List[GpuType] = none):
        self.accelerated_networking = accelerated_networking
        self.disk_performance = disk_performance
        self.architectures = architectures
        self.exclude_series = exclude_series
        self.max_memory_gi_b = max_memory_gi_b
        self.max_v_cpu = max_v_cpu
        self.min_memory_gi_b = min_memory_gi_b
        self.min_v_cpu = min_v_cpu
        self.min_gpu = min_gpu
        self.max_gpu = max_gpu
        self.min_data = min_data
        self.min_n_i_cs = min_n_i_cs
        self.series = series
        self.vm_types = vm_types
        self.gpu_types = gpu_types


class VmSizes:
    """
    # Arguments
    filters: Filters
    """

    def __init__(
            self,
            filters: Filters = none):
        self.filters = filters
# endregion


class VirtualNodeGroupTemplate:
    """
    # Arguments
    name: str
    ocean_id: str 
    auto_scale: AutoScale
    availability_zones: List[str]
    labels: dict
    node_count_limits: NodeCountLimits
    node_pool_properties: NodePoolProperties
    strategy: Strategy
    tags: dict
    taints: List[Taint]
    vm_sizes: VmSizes
    """

    def __init__(
            self,
            name: str = none,
            ocean_id: str = none,
            auto_scale: AutoScale = none,
            availability_zones: List[str] = none,
            labels: dict = none,
            node_count_limits: NodeCountLimits = none,
            node_pool_properties: NodePoolProperties = none,
            strategy: Strategy = none,
            tags: dict = none,
            taints: List[Taint] = none,
            vm_sizes: VmSizes = none):
        self.name = name
        self.ocean_id = ocean_id
        self.auto_scale = auto_scale
        self.availability_zones = availability_zones
        self.labels = labels
        self.node_count_limits = node_count_limits
        self.node_pool_properties = node_pool_properties
        self.strategy = strategy
        self.tags = tags
        self.taints = taints
        self.vm_sizes = vm_sizes

# endregion

# region Ocean


class Ocean:
    """
    # Arguments 
    aks: Aks
    auto_scaler: AutoScaler
    controller_cluster_id: str
    health: Health
    name: str
    scheduling: Scheduling
    virtual_node_group_template: VirtualNodeGroupTemplate
    """

    def __init__(
            self,
            aks: Aks = none,
            auto_scaler: AutoScaler = none,
            controller_cluster_id: str = none,
            health: Health = none,
            name: str = none,
            scheduling: Scheduling = none,
            virtual_node_group_template: VirtualNodeGroupTemplate = none):
        self.aks = aks
        self.auto_scaler = auto_scaler
        self.controller_cluster_id = controller_cluster_id
        self.health = health
        self.name = name
        self.scheduling = scheduling
        self.virtual_node_group_template = virtual_node_group_template
# endregion

# region OceanRequest


class OceanRequest:
    def __init__(self, ocean: Ocean):
        self.cluster = ocean

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion

# region VNGRequest


class VNGRequest:
    def __init__(self, vng: VirtualNodeGroupTemplate):
        self.virtual_node_group = vng

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion

# region LaunchNewNodesRequest


class PreferredLifecycle(Enum):
    spot = "Spot"
    regular = "Regular"


class LaunchNewNodes:
    """
    # Arguments
    adjustment: int
    applicable_vm_sizes: List[str]
    availability_zones: List[str]
    min_cores_per_node: int
    min_memory_gi_b_per_node: float
    ocean_id: str
    preferred_lifecycle: PreferredLifecycle
    vng_ids: List[str]
    """

    def __init__(
            self,
            adjustment: int = none,
            applicable_vm_sizes: List[str] = none,
            availability_zones: List[str] = none,
            min_cores_per_node: int = none,
            min_memory_gi_b_per_node: float = none,
            ocean_id: str = none,
            preferred_lifecycle: PreferredLifecycle = none,
            vng_ids: List[str] = none):
        self.adjustment = adjustment
        self.applicable_vm_sizes = applicable_vm_sizes
        self.availability_zones = availability_zones
        self.min_cores_per_node = min_cores_per_node
        self.min_memory_gi_b_per_node = min_memory_gi_b_per_node
        self.ocean_id = ocean_id
        self.preferred_lifecycle = preferred_lifecycle
        self.vng_ids = vng_ids


class LaunchNewNodesRequest:
    def __init__(self, launch_new_nodes: LaunchNewNodes):
        self.adjustment = launch_new_nodes.adjustment
        self.applicable_vm_sizes = launch_new_nodes.applicable_vm_sizes
        self.availability_zones = launch_new_nodes.availability_zones
        self.min_cores_per_node = launch_new_nodes.min_cores_per_node
        self.min_memory_gi_b_per_node = launch_new_nodes.min_memory_gi_b_per_node
        self.ocean_id = launch_new_nodes.ocean_id
        self.preferred_lifecycle = launch_new_nodes.preferred_lifecycle
        self.vng_ids = launch_new_nodes.vng_ids

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion

# region AggregatedClusterCosts


class Type(Enum):
    label = "label"
    annotation = "annotation"


class Operator(Enum):
    equals = "equals"
    not_equals = "notEquals"
    exists = "exists"
    not_exists = "notExists"


class Attribute:
    """
    # Arguments
    type: Type
    key: str
    operator: Operator
    value: str
    """

    def __init__(
            self,
            type: Type = none,
            key: str = none,
            operator: Operator = none,
            value: str = none):
        self.type = type
        self.key = key
        self.operator = operator
        self.value = value


class AllMatch:
    """
    # Arguments
    all_match:  List[Attribute]
    """

    def __init__(
            self,
            all_match:  List[Attribute] = none):
        self.all_match = all_match


class Conditions:
    """
    # Arguments
    any_match: List[AllMatch]
    """

    def __init__(
            self,
            any_match: List[AllMatch] = none):
        self.any_match = any_match


class Scope(Enum):
    namespace = "namespace"
    resource = "resource"


class Filter:
    """
    # Arguments
    conditions: Conditions
    scope: Scope
    """

    def __init__(
            self,
            conditions: Conditions = none,
            scope: Scope = none):
        self.conditions = conditions
        self.scope = scope


class GroupBy(Enum):
    namespace = "namespace"
    namespace_label = "namespace.label.${labelKey}"
    resource_label = "resource.label.${labelKey}"
    namespace_annotation = "namespace.annotation.${annotationKey}"
    resource_annotation = "resource.annotation.${annotationKey}"


class AggregatedClusterCosts:
    """
    # Arguments
    end_time: str
    filter: Filter
    group_by: GroupBy
    start_time: str
    """

    def __init__(
            self,
            end_time: str = none,
            filter: Filter = none,
            group_by: GroupBy = GroupBy.namespace.value,
            start_time: str = none):
        self.end_time = end_time
        self.filter = filter
        self.group_by = group_by
        self.start_time = start_time


class AggregatedClusterCostRequest:
    def __init__(self, aggregated_cluster_costs: AggregatedClusterCosts = none):
        self.end_time = aggregated_cluster_costs.end_time
        self.start_time = aggregated_cluster_costs.start_time
        self.group_by = aggregated_cluster_costs.group_by
        self.filter = aggregated_cluster_costs.filter

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion


class Roll:
    """
    # Arguments
    comment: str
    batch_size_percentage: int
    batch_min_healthy_percentage: int
    respect_pdb: bool
    node_pool_names: List[str]
    vng_ids: List[str]
    respect_restrict_scale_down: bool
    node_names: List[str]

    """

    def __init__(
            self,
            comment: str = none,
            batch_size_percentage: int = none,
            batch_min_healthy_percentage: int = none,
            respect_pdb: bool = none,
            node_pool_names: List[str] = none,
            vng_ids: List[str] = none,
            respect_restrict_scale_down: bool = none,
            node_names: List[str] = none):
        self.comment = comment
        self.batch_size_percentage = batch_size_percentage
        self.batch_min_healthy_percentage = batch_min_healthy_percentage
        self.respect_pdb = respect_pdb
        self.node_pool_names = node_pool_names
        self.vng_ids = vng_ids
        self.respect_restrict_scale_down = respect_restrict_scale_down
        self.node_names = node_names


class ClusterRollInitiateRequest:
    def __init__(self, roll: Roll = none):
        self.comment = roll.comment
        self.batch_size_percentage = roll.batch_size_percentage
        self.batch_min_healthy_percentage = roll.batch_min_healthy_percentage
        self.respect_pdb = roll.respect_pdb
        self.node_pool_names = roll.node_pool_names
        self.vng_ids = roll.vng_ids
        self.respect_restrict_scale_down = roll.respect_restrict_scale_down
        self.node_names = roll.node_names

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

# region Migration


class Migration:
    """
    # Arguments
    node_names: List[str]
    node_pool_names: List[str]
    batch_size_percentage: int
    batch_min_healthy_percentage: int
    comment: str
    respect_pdb: bool
    respect_restrict_scale_down: bool
    should_evict_standalone_pods: bool
    should_terminate_nodes: bool
    """

    def __init__(
            self,
            node_names: List[str] = none,
            node_pool_names: List[str] = none,
            batch_size_percentage: int = none,
            batch_min_healthy_percentage: int = none,
            comment: str = none,
            respect_pdb: bool = none,
            respect_restrict_scale_down: bool = none,
            should_evict_standalone_pods: bool = none,
            should_terminate_nodes: bool = none,
    ):
        self.node_names = node_names
        self.node_pool_names = node_pool_names
        self.batch_size_percentage = batch_size_percentage
        self.batch_min_healthy_percentage = batch_min_healthy_percentage
        self.comment = comment
        self.respect_pdb = respect_pdb
        self.respect_restrict_scale_down = respect_restrict_scale_down
        self.should_evict_standalone_pods = should_evict_standalone_pods
        self.should_terminate_nodes = should_terminate_nodes


class MigrationRequest:
    def __init__(self, migration: Migration):
        self.node_names = migration.node_names
        self.node_pool_names = migration.node_pool_names
        self.batch_size_percentage = migration.batch_size_percentage
        self.batch_min_healthy_percentage = migration.batch_min_healthy_percentage
        self.comment = migration.comment
        self.respect_pdb = migration.respect_pdb
        self.respect_restrict_scale_down = migration.respect_restrict_scale_down
        self.should_evict_standalone_pods = migration.should_evict_standalone_pods
        self.should_terminate_nodes = migration.should_terminate_nodes

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion


class DetachNodes:
    """
    # Arguments
    node_names_to_detach: List[str]
    ocean_id: str
    """
    def __init__(self,
                 node_names_to_detach: List[str] = none,
                 ocean_id: str = none):
        self.node_names_to_detach = node_names_to_detach
        self.ocean_id = ocean_id


class DetachNodesRequest:
    def __init__(self, detach_nodes_obj: DetachNodes):
        self.node_names_to_detach = detach_nodes_obj.node_names_to_detach
        self.ocean_id = detach_nodes_obj.ocean_id

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class RightSizingFilter:
    """
    # Arguments
    attribute: Attribute
    namespaces: List[str]
    """
    def __init__(self,
                 attribute: Attribute = none,
                 namespaces: List[str] = none):
        self.attribute = attribute
        self.namespaces = namespaces


class FetchRightSizingRequest:
    def __init__(self, filter: RightSizingFilter):
        self.filter = filter

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)



import json

from spotinst_sdk2.client import Client

import spotinst_sdk2.models.managed_instance.aws as aws_managed_instance

none = "d3043820717d74d9a17694c176d39733"


class ManagedInstanceAwsClient(Client):
    __base_mi_url = "/aws/ec2/managedInstance"
    ENTITY_NAME = 'managedInstance'

    def create_managed_instance(self, managed_instance: aws_managed_instance.ManagedInstance):
        """
        Create a Managed Instance

        # Arguments
        managed instance (ManagedInstance): ManagedInstance Object

        # Returns
        (Object): ManagedInstance API response
        """
        req = aws_managed_instance.ManagedInstanceCreationRequest(
            managed_instance)

        req_json_str = req.toJSON()
        req_exclude_missing = self.exclude_missing(json.loads(req_json_str))

        formatted_mi_dict = self.convert_json(
            req_exclude_missing, self.underscore_to_camel)

        body_json = json.dumps(formatted_mi_dict)

        self.print_output(body_json)

        mi_response = self.send_post(
            body=body_json,
            url=self.__base_mi_url,
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            mi_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def get_managed_instance(self, managed_instance_id: str):
        """
        Get a Managed Instance

        # Arguments
        managed_instance_id(String): Managed Instance ID

        # Returns
        (Object): ManagedInstance API response
        """
        geturl = self.__base_mi_url + "/" + managed_instance_id
        result = self.send_get(url=geturl, entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_managed_instances(self):
        """
        Get all Managed Instance

        # Returns
        List: List of ManagedInstance API response
        """
        content = self.send_get(
            url=self.__base_mi_url,
            entity_name=self.ENTITY_NAME)
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def update_managed_instance(self, managed_instance_id: str,
                                managed_instance_update: aws_managed_instance.ManagedInstance):
        """
        Update a Managed Instance

        # Arguments
        managed_instance_id(String): Managed Instance ID
        managed_instance_update(ManagedInstance): ManagedInstance Object

        # Returns
        (Object): ManagedInstance API response
        """
        group = aws_managed_instance.ManagedInstanceUpdateRequest(
            managed_instance_update)

        excluded_group_update_dict = self.exclude_missing(
            json.loads(group.toJSON()))

        formatted_group_update_dict = self.convert_json(
            excluded_group_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_update_dict)

        self.print_output(body_json)

        req_url = self.__base_mi_url + "/" + managed_instance_id

        group_response = self.send_put(
            body=body_json,
            url=req_url,
            entity_name=self.ENTITY_NAME,
        )

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def delete_managed_instance(self, managed_instance_id: str,
                                deallocation_config: aws_managed_instance.DeallocationConfig = none,
                                ami_backup: aws_managed_instance.AmiBackup = none):
        """
        Delete a Managed Instance

        # Arguments
        managed_instance_id(String): Managed Instance ID
        deallocation_config(DeallocationConfig): DeallocationConfig object
        ami_backup(AmiBackup): AmiBackup object

        # Returns
        (Object): ManagedInstance API response
        """
        req_url = self.__base_mi_url + "/" + managed_instance_id

        deletion_request = aws_managed_instance.ManagedInstanceDeletionRequest(
            deallocation_config, ami_backup)

        req_exclude_missing = self.exclude_missing(
            json.loads(deletion_request.toJSON()))

        formatted_deletion_dict = self.convert_json(
            req_exclude_missing, self.underscore_to_camel)

        body_json = json.dumps(formatted_deletion_dict)

        return self.send_delete_with_body(body=body_json, url=req_url, entity_name=self.ENTITY_NAME)

    def recycle_managed_instance(self, managed_instance_id: str):
        """
        Recycle a Managed Instance

        # Arguments
        managed_instance_id(String): Managed Instance ID

        # Returns
        (Object): ManagedInstance API response
        """
        put_url = self.__base_mi_url + "/" + managed_instance_id + "/" + "recycle"
        result = self.send_put(url=put_url, entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]

    def pause_managed_instance(self, managed_instance_id: str):
        """
        Pause a Managed Instance

        # Arguments
        managed_instance_id(String): Managed Instance ID

        # Returns
        (Object): ManagedInstance API response
        """
        put_url = self.__base_mi_url + "/" + managed_instance_id + "/" + "pause"
        result = self.send_put(url=put_url, entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]

    def resume_managed_instance(self, managed_instance_id: str):
        """
        Resume a Managed Instance

        # Arguments
        managed_instance_id(String): Managed Instance ID

        # Returns
        (Object): ManagedInstance API response
        """
        put_url = self.__base_mi_url + "/" + managed_instance_id + "/" + "resume"
        result = self.send_put(url=put_url, entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]

    def get_managed_instance_status(self, managed_instance_id: str):
        """
        Get Managed Instance Status

        # Arguments
        managed_instance_id(String): Managed Instance ID

        # Returns
        (Object): ManagedInstance API response
        """
        geturl = self.__base_mi_url + "/" + managed_instance_id + "/" + "status"
        result = self.send_get(url=geturl, entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_managed_instance_costs(self, managed_instance_id: str, from_date: str, to_date: str, aggregation_period: str = None):
        """
        Get Managed Instance Costs

        # Arguments
        managed_instance_id(String): Managed Instance ID
        from_date (String): Get Cost on and after this Date
        to_date (String): Get Cost on and before this Date
        aggregation_period (String): Data values following either a date format or Unix seconds Timestamp

        # Returns
        (Object): ManagedInstance API response
        """
        geturl = self.__base_mi_url + "/" + managed_instance_id + "/" + "costs"

        query_params = dict(toDate=to_date, fromDate=from_date,
                            aggregrationPeriod=aggregation_period)

        result = self.send_get(
            url=geturl, entity_name=self.ENTITY_NAME, query_params=query_params)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_volume_in_managed_instance(self, managed_instance_id: str, volume_id: str):
        """
        Delete Volume in a Managed Instance

        # Arguments
        managed_instance_id(String): Managed Instance ID
        volume_id(String): Volume ID

        # Returns
        (Object): ManagedInstance API response
        """
        req_url = self.__base_mi_url + "/" + \
            managed_instance_id + "/" + "volume" + "/" + volume_id

        return self.send_delete(url=req_url, entity_name=self.ENTITY_NAME)

    def update_managed_instance_states(self, update_manage_instance_states_list: list):
        """
        Update a Managed Instance States

        # Arguments
        update_manage_instance_state_list(List): Update Manage Instance List

        # Returns
        (Object): ManagedInstance API response
        """
        put_url = self.__base_mi_url + "/" + "state"

        update_state_request = aws_managed_instance.ManagedInstanceUpdateStatesRequest(
            update_manage_instance_states_list)

        excluded_state_update_dict = self.exclude_missing(
            json.loads(update_state_request.toJSON()))

        formatted_state_update_dict = self.convert_json(
            excluded_state_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_state_update_dict)

        group_response = self.send_put(
            body=body_json,
            url=put_url,
            entity_name=self.ENTITY_NAME,
        )

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        return formatted_response['response']

    def import_multiple_instances_from_aws(self, migrations_list: list):
        """
        Import Multiple Instances From AWS

        # Arguments
        migrations_list (List): ManagedInstanceMigration List

        # Returns
        (Object): ManagedInstanceMigrations API response
        """
        req = aws_managed_instance.ManagedInstanceBulkMigrationRequest(
            migrations_list)

        post_url = self.__base_mi_url + "/" + "migration" + "/" + "bulk"

        req_exclude_missing = self.exclude_missing(json.loads(req.toJSON()))

        formatted_mi_dict = self.convert_json(
            req_exclude_missing, self.underscore_to_camel)

        body_json = json.dumps(formatted_mi_dict)

        mi_response = self.send_post(
            body=body_json,
            url=post_url,
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            mi_response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_multiple_instances_migration_status(self, migration_ids: list):
        """
        Import Multiple Instances From AWS

        # Arguments
        migration_ids (List): List of ManagedInstanceMigrationIds 

        # Returns
        (Object): ManagedInstanceMigrations API response
        """
        req = aws_managed_instance.ManagedInstanceBulkMigrationStatusRequest(
            migration_ids)

        post_url = self.__base_mi_url + "/" + "migration" + "/" + "search"

        req_exclude_missing = self.exclude_missing(json.loads(req.toJSON()))

        formatted_mi_dict = self.convert_json(
            req_exclude_missing, self.underscore_to_camel)

        body_json = json.dumps(formatted_mi_dict)

        mi_response = self.send_post(
            body=body_json,
            url=post_url,
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            mi_response, self.camel_to_underscore)

        return formatted_response["response"]["items"]



import json

from spotinst_sdk2.client import Client
import spotinst_sdk2.models.subscription as spotinst_event_subscription


class SubscriptionClient(Client):
    __base_event_subscription_url = "/events/subscription"

    # region Event Subscription
    def create_event_subscription(self, subscription):
        """
        Create an event subscription 

        # Arguments
        subscription (Subscription): Subscription Object

        # Returns
        (Object): Subscription API response 
        """
        subscription = spotinst_event_subscription.SubscriptionRequest(
            subscription)

        excluded_group_dict = self.exclude_missing(
            json.loads(subscription.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_event_subscription_url,
            entity_name='subscription')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def update_event_subscription(self, subscription_id, subscription):
        """
        Update an existing event subscription 

        # Arguments
        subscription_id (String): Subscription id
        subscription (Subscription): Subscription Object

        # Returns
        (Object): Subscription API response 
        """
        subscription = spotinst_event_subscription.SubscriptionRequest(
            subscription)

        excluded_group_dict = self.exclude_missing(
            json.loads(subscription.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_put(
            body=body_json,
            url=self.__base_event_subscription_url +
            "/" + subscription_id,
            entity_name='subscription')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        retVal = formatted_response["response"]

        return retVal

    def get_all_event_subscription(self):
        """
        Get all Subscription in account

        # Returns
        (Object): Subscription API response 
        """
        response = self.send_get(
            url=self.__base_event_subscription_url,
            entity_name="subscription"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal

    def get_event_subscription(self, subscription_id):
        """
        Get an existing event subscription json

        # Arguments
        subscription_id (String): Subscription id

        # Returns
        (Object): Subscription API response 
        """
        response = self.send_get(
            url=self.__base_event_subscription_url +
            "/" + subscription_id,
            entity_name="subscription"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def delete_event_subscription(self, subscription_id):
        """
        Delete an event subscription

        # Arguments
        subscription_id (String): Subscription id

        # Returns
        (Object): subscription response 
        """
        response = self.send_delete(
            url=self.__base_event_subscription_url +
            "/" + subscription_id,
            entity_name="subscription"
        )

        return response
    # end region



import os
import configparser
import yaml
from spotinst_sdk2.client import SpotinstClientException

VAR_SPOTINST_SHARED_CREDENTIALS_FILE = 'SPOTINST_SHARED_CREDENTIALS_FILE'
VAR_SPOTINST_PROFILE = 'SPOTINST_PROFILE'
VAR_SPOTINST_TOKEN = 'SPOTINST_TOKEN'
VAR_SPOTINST_ACCOUNT = 'SPOTINST_ACCOUNT'

DEFAULT_PROFILE = 'default'
DEFAULT_CREDENTIALS_FILE = os.path.join(
    os.path.expanduser("~"), '.spotinst', 'credentials')


class Session:
    def __init__(self,
                 base_url=None,
                 auth_token=None,
                 account_id=None,
                 profile=None,
                 credentials_file=None):

        if base_url is None:
            self.base_url = "https://api.spotinst.io"
        else:
            self.base_url = base_url

        if auth_token is None:
            self.load_credentials(profile, credentials_file)
        else:
            self.auth_token = auth_token
            self.account_id = account_id

    def load_credentials(self, profile, credentials_file):
        self.account_id = os.environ.get(VAR_SPOTINST_ACCOUNT, None)
        self.auth_token = os.environ.get(VAR_SPOTINST_TOKEN, None)

        if not self.auth_token:
            if not profile:
                profile = os.environ.get(VAR_SPOTINST_PROFILE, DEFAULT_PROFILE)

            if not credentials_file:
                credentials_file = os.environ.get(
                    VAR_SPOTINST_SHARED_CREDENTIALS_FILE,
                    DEFAULT_CREDENTIALS_FILE)

            with open(credentials_file, 'r', encoding='utf-8') as credentials_file:
                self.__load_credentials_yaml(profile, credentials_file)
                if not self.auth_token:
                    self.__load_credentials_ini(profile, credentials_file)

            if not self.auth_token:
                raise SpotinstClientException(
                    "failed to load credentials", "ERROR")

    def __load_credentials_yaml(self, profile, credentials_file):
        try:
            config = yaml.load(credentials_file, Loader=yaml.SafeLoader)
            if config:
                self.account_id = config.get(profile, {}).get("account", None)
                self.auth_token = config.get(profile, {}).get("token", None)
        except (yaml.YAMLError, KeyError):
            return

    def __load_credentials_ini(self, profile, credentials_file):
        try:
            config = configparser.ConfigParser()
            config.read(credentials_file.name)
            if config[profile]:
                self.account_id = config[profile]["account"]
                self.auth_token = config[profile]["token"]
        except (configparser.NoSectionError, configparser.NoOptionError):
            return



import json
from enum import Enum
from typing import List

none = "d3043820717d74d9a17694c176d39733"


class Down:
    """
    # Arguments
    evaluation_periods: int
    max_scale_down_percentage: int
    """

    def __init__(self,
                 evaluation_periods: int = none,
                 max_scale_down_percentage: int = none):
        self.evaluation_periods = evaluation_periods
        self.max_scale_down_percentage = max_scale_down_percentage


class Headroom:
    """
    # Arguments
    cpu_per_unit: int
    memory_per_unit: int
    num_of_units: int
    """

    def __init__(self,
                 cpu_per_unit: int = none,
                 memory_per_unit: int = none,
                 num_of_units: int = none):
        self.cpu_per_unit = cpu_per_unit
        self.memory_per_unit = memory_per_unit
        self.num_of_units = num_of_units


class ResourceLimits:
    """
    # Arguments
    max_memory_gib: int
    max_v_cpu: int
    """

    def __init__(self,
                 max_memory_gib: int = none,
                 max_v_cpu: int = none):
        self.max_memory_gib = max_memory_gib
        self.max_v_cpu = max_v_cpu


class AutoScaler:
    """
    # Arguments
    auto_headroom_percentage: int
    cooldown: int
    down: Down
    enable_automatic_and_manual_headroom: bool
    headroom: Headroom
    is_auto_config: bool
    is_enabled: bool
    resource_limits: ResourceLimits
    should_scale_down_non_service_tasks: bool
    """

    def __init__(self,
                 auto_headroom_percentage: int = none,
                 cooldown: int = none,
                 down: Down = none,
                 enable_automatic_and_manual_headroom: bool = none,
                 headroom: Headroom = none,
                 is_auto_config: bool = none,
                 is_enabled: bool = none,
                 resource_limits: ResourceLimits = none,
                 should_scale_down_non_service_tasks: bool = none):
        self.auto_headroom_percentage = auto_headroom_percentage
        self.cooldown = cooldown
        self.down = down
        self.enable_automatic_and_manual_headroom = enable_automatic_and_manual_headroom
        self.headroom = headroom
        self.is_auto_config = is_auto_config
        self.is_enabled = is_enabled
        self.resource_limits = resource_limits
        self.should_scale_down_non_service_tasks = should_scale_down_non_service_tasks


class Capacity:
    """
    # Arguments
    maximum: int
    minimum: int
    target: int
    """

    def __init__(self,
                 maximum: int = none,
                 minimum: int = none,
                 target: int = none):
        self.maximum = maximum
        self.minimum = minimum
        self.target = target


class Architecture(Enum):
    i386 = "i386"
    x86_64 = "x86_64"
    arm64 = "arm64"


class Category(Enum):
    accelerated_computing = "Accelerated_computing"
    compute_optimized = "Compute_optimized"
    general_purpose = "General_purpose"
    memory_optimized = "Memory_optimized"
    storage_optimized = "Storage_optimized"


class DiskType(Enum):
    nvme = "NVMe"
    ebs = "EBS"
    ssd = "SSD"
    hdd = "HDD"


class Hypervisor(Enum):
    nitro = "nitro"
    xen = "xen"


class RootDeviceType(Enum):
    ebs = "ebs"
    instance_store = "instance-store"


class VirtualizationType(Enum):
    hvm = "hvm"
    paravirtual = "paravirtual"


class InstanceTypesFilters:
    """
    # Arguments
    min_vcpu: int
    max_vcpu: int
    min_memory_gi_b: float
    max_memory_gi_b: float
    min_gpu: int
    max_gpu: int
    include_families: List[str]
    exclude_families: List[str]
    exclude_metal: bool
    is_ena_supported: bool
    architectures: List[Architecture]
    virtualization_types: List[VirtualizationType]
    categories: List[Category]
    min_enis: int
    disk_types: List[DiskType]
    hypervisor: List[Hypervisor]
    root_device_types: List[RootDeviceType]
    min_network_performance: int
    max_network_performance: int
    """

    def __init__(self,
                 min_vcpu: int = none,
                 max_vcpu: int = none,
                 min_memory_gi_b: float = none,
                 max_memory_gi_b: float = none,
                 min_gpu: int = none,
                 max_gpu: int = none,
                 include_families: List[str] = none,
                 exclude_families: List[str] = none,
                 exclude_metal: bool = none,
                 is_ena_supported: bool = none,
                 architectures: List[Architecture] = none,
                 virtualization_types: List[VirtualizationType] = none,
                 categories: List[Category] = none,
                 min_enis: int = none,
                 disk_types: List[DiskType] = none,
                 hypervisor: List[Hypervisor] = none,
                 root_device_types: List[RootDeviceType] = none,
                 min_network_performance: int = none,
                 max_network_performance: int = none):
        self.min_vcpu = min_vcpu
        self.max_vcpu = max_vcpu
        self.min_memory_gi_b = min_memory_gi_b
        self.max_memory_gi_b = max_memory_gi_b
        self.min_gpu = min_gpu
        self.max_gpu = max_gpu
        self.include_families = include_families
        self.exclude_families = exclude_families
        self.exclude_metal = exclude_metal
        self.is_ena_supported = is_ena_supported
        self.architectures = architectures
        self.virtualization_types = virtualization_types
        self.categories = categories
        self.min_enis = min_enis
        self.disk_types = disk_types
        self.hypervisor = hypervisor
        self.root_device_types = root_device_types
        self.min_network_performance = min_network_performance
        self.max_network_performance = max_network_performance


class InstanceTypes:
    """
    # Arguments
    blacklist: List[str]
    filters: InstanceTypesFilters
    whitelist: List[str]
    """

    def __init__(self,
                 blacklist: List[str] = none,
                 filters: InstanceTypesFilters = none,
                 whitelist: List[str] = none):
        self.blacklist = blacklist
        self.filters = filters
        self.whitelist = whitelist


class DynamicVolumeSize:
    """
    # Arguments
    base_size: int
    resource: str
    size_per_resource_unit: int
    """

    def __init__(self,
                 base_size: int = none,
                 resource: str = none,
                 size_per_resource_unit: int = none):
        self.base_size = base_size
        self.resource = resource
        self.size_per_resource_unit = size_per_resource_unit


class EBS:
    """
    # Arguments
    delete_on_termination: bool
    dynamic_volume_size: DynamicVolumeSize
    encrypted: bool
    iops: int
    kms_key_id: str
    snapshot_id: str
    throughput: int
    volume_size: int
    volume_type: str
    """

    def __init__(self,
                 delete_on_termination: bool = none,
                 dynamic_volume_size: DynamicVolumeSize = none,
                 encrypted: bool = none,
                 iops: int = none,
                 kms_key_id: str = none,
                 snapshot_id: str = none,
                 throughput: int = none,
                 volume_size: int = none,
                 volume_type: str = none):
        self.delete_on_termination = delete_on_termination
        self.dynamic_volume_size = dynamic_volume_size
        self.encrypted = encrypted
        self.iops = iops
        self.kms_key_id = kms_key_id
        self.snapshot_id = snapshot_id
        self.throughput = throughput
        self.volume_size = volume_size
        self.volume_type = volume_type


class BlockDeviceMapping:
    """
    # Arguments
    device_name: str
    ebs: EBS
    """

    def __init__(self,
                 device_name: str = none,
                 ebs: EBS = none):
        self.device_name = device_name
        self.ebs = ebs


class IamInstanceProfile:
    """
    # Arguments
    arn: str
    """

    def __init__(self,
                 arn: str = none):
        self.arn = arn


class HttpEndpoint(Enum):
    disabled = "disabled"
    enabled = "enabled"


class HttpTokens(Enum):
    optional = "optional"
    required = "required"


class InstanceMetadataOptions:
    """
    # Arguments
    http_endpoint: HttpEndpoint
    http_put_response_hop_limit: int
    http_tokens: HttpTokens
    """

    def __init__(self,
                 http_endpoint: HttpEndpoint = none,
                 http_put_response_hop_limit: int = none,
                 http_tokens: HttpTokens = none):
        self.http_endpoint = http_endpoint
        self.http_put_response_hop_limit = http_put_response_hop_limit
        self.http_tokens = http_tokens


class Tag:
    """
    # Arguments
    tag_key: str
    tag_value: str
    """

    def __init__(self,
                 tag_key: str = none,
                 tag_value: str = none):
        self.tag_key = tag_key
        self.tag_value = tag_value


class LaunchSpecification:
    """
    # Arguments
    associate_ipv6_address: bool
    associate_public_ip_address: bool
    block_device_mappings: List[BlockDeviceMapping]
    ebs_optimized: bool
    iam_instance_profile: IamInstanceProfile
    image_id: str
    instance_metadata_options: InstanceMetadataOptions
    monitoring: bool
    security_group_ids: List[str]
    tags: List[Tag]
    use_as_template_only: bool
    user_data: str
    key_pair: str
    """

    def __init__(self,
                 associate_ipv6_address: bool = none,
                 associate_public_ip_address: bool = none,
                 block_device_mappings: List[BlockDeviceMapping] = none,
                 ebs_optimized: bool = none,
                 iam_instance_profile: IamInstanceProfile = none,
                 image_id: str = none,
                 instance_metadata_options: InstanceMetadataOptions = none,
                 monitoring: bool = none,
                 security_group_ids: List[str] = none,
                 tags: List[Tag] = none,
                 use_as_template_only: bool = none,
                 user_data: str = none,
                 key_pair: str = none):
        self.associate_ipv6_address = associate_ipv6_address
        self.associate_public_ip_address = associate_public_ip_address
        self.block_device_mappings = block_device_mappings
        self.ebs_optimized = ebs_optimized
        self.iam_instance_profile = iam_instance_profile
        self.image_id = image_id
        self.instance_metadata_options = instance_metadata_options
        self.monitoring = monitoring
        self.security_group_ids = security_group_ids
        self.tags = tags
        self.use_as_template_only = use_as_template_only
        self.user_data = user_data
        self.key_pair = key_pair


class OptimizeImages:
    """
    # Arguments
    perform_at: str
    should_optimize_ecs_ami: bool
    time_windows: List[str]
    """

    def __init__(self,
                 perform_at: str = none,
                 should_optimize_ecs_ami: bool = none,
                 time_windows: List[str] = none):
        self.perform_at = perform_at
        self.should_optimize_ecs_ami = should_optimize_ecs_ami
        self.time_windows = time_windows


class Compute:
    """
    # Arguments
    instance_types: InstanceTypes
    launch_specification: LaunchSpecification
    optimize_images: OptimizeImages
    subnet_ids: List[str]
    """

    def __init__(self,
                 instance_types: InstanceTypes = none,
                 launch_specification: LaunchSpecification = none,
                 optimize_images: OptimizeImages = none,
                 subnet_ids: List[str] = none):
        self.instance_types = instance_types
        self.launch_specification = launch_specification
        self.optimize_images = optimize_images
        self.subnet_ids = subnet_ids


class S3Bucket:
    """
    # Arguments
    id: str
    """

    def __init__(self,
                 id: str = none):
        self.id = id


class LoggingConfiguration:
    """
    # Arguments
    s3: S3Bucket
    """

    def __init__(self,
                 s3: S3Bucket = none):
        self.s3 = s3


class Logging:
    """
    # Arguments
    export: LoggingConfiguration
    """

    def __init__(self,
                 export: LoggingConfiguration = none):
        self.export = export


class ShutdownHours:
    """
    # Arguments
    is_enabled: bool
    time_windows: List[str]
    """

    def __init__(self,
                 is_enabled: bool = none,
                 time_windows: List[str] = none):
        self.is_enabled = is_enabled
        self.time_windows = time_windows


class ClusterRoll:
    """
    # Arguments
    batch_min_healthy_percentage: int
    batch_size_percentage: int
    comment: str
    """

    def __init__(self,
                 batch_min_healthy_percentage: int = none,
                 batch_size_percentage: int = none,
                 comment: str = none):
        self.batch_min_healthy_percentage = batch_min_healthy_percentage
        self.batch_size_percentage = batch_size_percentage
        self.comment = comment


class Parameters:
    """
    # Arguments
    cluster_roll: ClusterRoll
    """

    def __init__(self,
                 cluster_roll: ClusterRoll = none):
        self.cluster_roll = cluster_roll


class SchedulingTask:
    """
    # Arguments
    cron_expression: str
    is_enabled: bool
    parameters: Parameters
    task_type: str
    """

    def __init__(self,
                 cron_expression: str = none,
                 is_enabled: bool = none,
                 parameters: Parameters = none,
                 task_type: str = none):
        self.cron_expression = cron_expression
        self.is_enabled = is_enabled
        self.parameters = parameters
        self.task_type = task_type


class Scheduling:
    """
    # Arguments
    shutdown_hours: ShutdownHours
    tasks: List[SchedulingTask]
    """

    def __init__(self,
                 shutdown_hours: ShutdownHours = none,
                 tasks: List[SchedulingTask] = none):
        self.shutdown_hours = shutdown_hours
        self.tasks = tasks


class AvailabilityVsCost(Enum):
    cost_oriented = "costOriented"
    balanced = "balanced"
    cheapest = "cheapest"


class ClusterOrientation:
    """
    # Arguments
    availability_vs_cost: AvailabilityVsCost
    """

    def __init__(self,
                 availability_vs_cost: AvailabilityVsCost = none):
        self.availability_vs_cost = availability_vs_cost


class Strategy:
    """
    # Arguments
    cluster_orientation: ClusterOrientation
    draining_timeout: int
    fallback_to_od: bool
    spot_percentage: int
    utilize_commitments: bool
    utilize_reserved_instances: bool
    """

    def __init__(self,
                 cluster_orientation: ClusterOrientation = none,
                 draining_timeout: int = none,
                 fallback_to_od: bool = none,
                 spot_percentage: int = none,
                 utilize_commitments: bool = none,
                 utilize_reserved_instances: bool = none):
        self.cluster_orientation = cluster_orientation
        self.draining_timeout = draining_timeout
        self.fallback_to_od = fallback_to_od
        self.spot_percentage = spot_percentage
        self.utilize_commitments = utilize_commitments
        self.utilize_reserved_instances = utilize_reserved_instances


class Ocean:
    """
    # Arguments
    auto_scaler: AutoScaler
    capacity: Capacity
    cluster_name: str
    compute: Compute
    logging: Logging
    name: str
    region: str
    scheduling: Scheduling
    strategy: Strategy
    """

    def __init__(self,
                 auto_scaler: AutoScaler = none,
                 capacity: Capacity = none,
                 cluster_name: str = none,
                 compute: Compute = none,
                 logging: Logging = none,
                 name: str = none,
                 region: str = none,
                 scheduling: Scheduling = none,
                 strategy: Strategy = none):
        self.auto_scaler = auto_scaler
        self.capacity = capacity
        self.cluster_name = cluster_name
        self.compute = compute
        self.logging = logging
        self.name = name
        self.region = region
        self.scheduling = scheduling
        self.strategy = strategy


class OceanRequest:
    def __init__(self, ocean: Ocean):
        self.cluster = ocean

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ImportClusterConfig:
    """
    # Arguments
    instance_id: str
    name: str
    region: str
    """

    def __init__(self,
                 instance_id: str = none,
                 name: str = none,
                 region: str = none):
        self.instance_id = instance_id
        self.name = name
        self.region = region


class ImportClusterRequest:
    def __init__(self, import_cluster_config: ImportClusterConfig):
        self.region = import_cluster_config.region
        self.name = import_cluster_config.name
        self.instance_id = import_cluster_config.instance_id

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class InstanceTypesFilterRequest:
    def __init__(self, filters: InstanceTypesFilters):
        self.filters = filters

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class RollConfig:
    """
    # Arguments
    batch_min_healthy_percentage: int
    batch_size_percentage: int
    comment: str
    instance_ids: List[str]
    launch_spec_ids: List[str]
    """

    def __init__(
            self,
            batch_min_healthy_percentage: int = none,
            batch_size_percentage: int = none,
            comment: str = none,
            launch_spec_ids: List[str] = none,
            instance_ids: List[str] = none):
        self.batch_min_healthy_percentage = batch_min_healthy_percentage
        self.batch_size_percentage = batch_size_percentage
        self.comment = comment
        self.instance_ids = instance_ids
        self.launch_spec_ids = launch_spec_ids


class ClusterRollInitiateRequest:
    def __init__(self, roll_config: RollConfig = none):
        self.roll = roll_config

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ClusterRollUpdateRequest:
    def __init__(self, status: str = none):
        self.roll = dict(status=status)

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class DetachInstancesConfig:
    """
    # Arguments
    instances_to_detach: List[str]
    should_decrement_target_capacity: bool
    should_terminate_instances: bool
    """

    def __init__(
            self,
            instances_to_detach: List[str] = none,
            should_decrement_target_capacity: bool = none,
            should_terminate_instances: bool = none):
        self.instances_to_detach = instances_to_detach
        self.should_decrement_target_capacity = should_decrement_target_capacity
        self.should_terminate_instances = should_terminate_instances


class DetachInstancesRequest:
    def __init__(self, detach_config: DetachInstancesConfig):
        self.instances_to_detach = detach_config.instances_to_detach
        self.should_decrement_target_capacity = detach_config.should_decrement_target_capacity
        self.should_terminate_instances = detach_config.should_terminate_instances

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class Attribute:
    """
    # Arguments
    key: str
    value: str
    """

    def __init__(self,
                 key: str = none,
                 value: str = none):
        self.key = key
        self.value = value


class AutoScale:
    """
    # Arguments
    headrooms: List[Headrooms]
    """

    def __init__(self,
                 headrooms: List[Headroom] = none):
        self.headrooms = headrooms


class Image:
    """
    # Arguments
    id: str
    """

    def __init__(self,
                 id: str = none):
        self.id = id


class VngTask:
    """
    # Arguments
    config: List[Headroom]
    cron_expression: str
    is_enabled: bool
    task_type: str
    """

    def __init__(self,
                 config: List[Headroom] = none,
                 cron_expression: str = none,
                 is_enabled: bool = none,
                 task_type: str = none):
        self.config = config
        self.cron_expression = cron_expression
        self.is_enabled = is_enabled
        self.task_type = task_type


class VngScheduling:
    """
    # Arguments
    tasks: List[VngTask]
    """

    def __init__(self,
                 tasks: List[VngTask]):
        self.tasks = tasks


class VirtualNodeGroup:
    """
    # Arguments
    attributes: List[Attribute]
    auto_scale: AutoScale
    block_device_mappings: List[BlockDeviceMapping]
    iam_instance_profile: IamInstanceProfile
    image_id: str
    images: List[Image]
    instance_metadata_options: InstanceMetadataOptions
    instance_types: List[str]
    name: str
    ocean_id: str
    preferred_spot_types: List[str]
    restrict_scale_down: bool
    scheduling: VngScheduling
    security_group_ids: List[str]
    strategy: Strategy
    subnet_ids: List[str]
    tags: List[Tag]
    user_data: str
    """

    def __init__(self,
                 attributes: List[Attribute] = none,
                 auto_scale: AutoScale = none,
                 block_device_mappings: List[BlockDeviceMapping] = none,
                 iam_instance_profile: IamInstanceProfile = none,
                 image_id: str = none,
                 images: List[Image] = none,
                 instance_metadata_options: InstanceMetadataOptions = none,
                 instance_types: List[str] = none,
                 name: str = none,
                 ocean_id: str = none,
                 preferred_spot_types: List[str] = none,
                 restrict_scale_down: bool = none,
                 scheduling: VngScheduling = none,
                 security_group_ids: List[str] = none,
                 strategy: Strategy = none,
                 subnet_ids: List[str] = none,
                 tags: List[Tag] = none,
                 user_data: str = none):
        self.attributes = attributes
        self.auto_scale = auto_scale
        self.block_device_mappings = block_device_mappings
        self.iam_instance_profile = iam_instance_profile
        self.image_id = image_id
        self.images = images
        self.instance_metadata_options = instance_metadata_options
        self.instance_types = instance_types
        self.name = name
        self.ocean_id = ocean_id
        self.preferred_spot_types = preferred_spot_types
        self.restrict_scale_down = restrict_scale_down
        self.scheduling = scheduling
        self.security_group_ids = security_group_ids
        self.strategy = strategy
        self.subnet_ids = subnet_ids
        self.tags = tags
        self.user_data = user_data


class VNGRequest:
    def __init__(self, vng: VirtualNodeGroup):
        self.launch_spec = vng

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ImportFargateToExistingOceanCluster:
    """
    # Arguments
    services: List[str]
    simple_new_service_names: bool
    """

    def __init__(self,
                 services: List[str] = none,
                 simple_new_service_names: bool = none):
        self.services = services
        self.simple_new_service_names = simple_new_service_names


class ImportFargateToExistingOceanClusterRequest:
    def __init__(self, import_config: ImportFargateToExistingOceanCluster):
        self.services = import_config.services
        self.simple_new_service_names = import_config.simple_new_service_names

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ImportFargateToNewOceanCluster:
    """
    # Arguments
    ecs_cluster_name: str
    key_pair: str
    ocean_cluster_name: str
    region: str
    services: List[str]
    tags: List[Tag]
    """

    def __init__(self,
                 ecs_cluster_name: str = none,
                 key_pair: str = none,
                 ocean_cluster_name: str = none,
                 region: str = none,
                 services: List[str] = none,
                 tags: List[Tag] = none):
        self.ecs_cluster_name = ecs_cluster_name
        self.key_pair = key_pair
        self.ocean_cluster_name = ocean_cluster_name
        self.region = region
        self.services = services
        self.tags = tags


class ImportFargateToNewOceanClusterRequest:
    def __init__(self, import_config: ImportFargateToNewOceanCluster):
        self.ecs_cluster_name = import_config.ecs_cluster_name
        self.key_pair = import_config.key_pair
        self.ocean_cluster_name = import_config.ocean_cluster_name
        self.region = import_config.region
        self.services = import_config.services
        self.tags = import_config.tags

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class LaunchInstancesRequest:
    def __init__(self, amount: int = none):
        self.launch_request = dict(amount=amount)

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)



import json

from spotinst_sdk2.client import Client

# region AWS imports
import spotinst_sdk2.models.elastigroup.aws as aws_elastigroup
import spotinst_sdk2.models.elastigroup.aws.stateful as aws_stateful
import spotinst_sdk2.models.elastigroup.aws.deployment as aws_deployment
import spotinst_sdk2.models.elastigroup.aws.deployment_action as aws_deployment_action
import spotinst_sdk2.models.elastigroup.aws.asg as aws_import
# endregion

# region GCP imports
import spotinst_sdk2.models.elastigroup.gcp as gcp_elastigroup
# endregion

# region Azure V3 imports
import spotinst_sdk2.models.elastigroup.azure_v3 as azure_v3_elastigroup


# endregion

# region AWS


class ElastigroupAwsClient(Client):
    __base_elastigroup_url = "/aws/ec2/group"
    __base_aws_url = "/aws/ec2"
    __base_stateful_url = "/aws/ec2/statefulMigrationGroup"

    # region Elastigroup
    def create_elastigroup(self, group, async_scale=None):
        """
        Create an elastigroup

        # Arguments
        group (Elastigroup): Elastigroup Object

        # Returns
        (Object): Elastigroup API response 
        """
        group = aws_elastigroup.ElastigroupCreationRequest(group)

        excluded_group_dict = self.exclude_missing(json.loads(group.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_post_with_params(
            body=body_json,
            url=self.__base_elastigroup_url,
            entity_name='elastigroup',
            user_query_params=dict(asyncScale=async_scale))

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def update_elastigroup(self, group_update, group_id, auto_apply_tags=None):
        """
        Update an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        group_update (Elastigroup): Elastigroup Object

        # Returns
        (Object): Elastigroup API response 
        """
        group = aws_elastigroup.ElastigroupUpdateRequest(group_update)

        excluded_group_update_dict = self.exclude_missing(
            json.loads(group.toJSON()))

        formatted_group_update_dict = self.convert_json(
            excluded_group_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_update_dict)

        group_response = self.send_put_with_params(
            body=body_json,
            url=self.__base_elastigroup_url + "/" + group_id,
            entity_name='elastigroup',
            user_query_params=dict(autoApplyTags=auto_apply_tags)
        )

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def delete_elastigroup(self, group_id):
        """
        Delete an elastigroup

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        delurl = self.__base_elastigroup_url + "/" + group_id
        response = self.send_delete(url=delurl, entity_name='elastigroup')
        return response

    def get_elastigroup(self, group_id):
        """
        Get an elastigroup

        # Arguments
        group_id(String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        geturl = self.__base_elastigroup_url + "/" + group_id
        result = self.send_get(url=geturl, entity_name='elastigroup')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_elastigroups(self):
        """
        Get all elastigroups

        # Returns
        (List): List of Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_elastigroup_url,
            entity_name='elastigroup')
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def scale_elastigroup_up(self, group_id, adjustment):
        """
        Scale up an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        adjustment (int): Ammount to scale group

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict({"adjustment": adjustment})
        content = self.send_put_with_params(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/scale/up",
            entity_name='elastigroup (scale up)',
            body=None,
            user_query_params=query_params)

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def scale_elastigroup_down(self, group_id, adjustment):
        """
        Scale down an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        adjustment (int): Ammount to scale group

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict({"adjustment": adjustment})
        content = self.send_put_with_params(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/scale/down",
            entity_name='elastigroup (scale down)',
            body=None,
            user_query_params=query_params)

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def delete_elastigroup_with_deallocation(self, group_id, stateful_deallocation):
        """
        Delete a stateful elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        stateful_deallocation (Deallocation): Deallocation Object

        # Returns
        (Object): Elastigroup API response 
        """
        delurl = self.__base_elastigroup_url + "/" + group_id

        deletion_request = aws_elastigroup.ElastigroupDeletionRequest(
            stateful_deallocation)

        excluded_deletion_dict = self.exclude_missing(
            json.loads(deletion_request.toJSON()))
        formatted_deletion_dict = self.convert_json(
            excluded_deletion_dict, self.underscore_to_camel)
        body_json = json.dumps(formatted_deletion_dict)

        response = self.send_delete_with_body(
            body=body_json, url=delurl, entity_name='elastigroup')

        return response

    def get_elastigroup_active_instances(self, group_id):
        """
        Get active instances of an elastigroup

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/status",
            entity_name='active instances')
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def get_elastigroup_activity(self, group_id, start_date):
        """
        Get elastigroup activity

        # Arguments
        group_id (String): Elastigroup ID
        start_date (String): Date when to start checking

        # Returns
        (Object) : Elastigroup API response 
        """
        query_params = self.build_query_params_with_input(
            {"fromDate": start_date})

        content = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/events",
            query_params=query_params,
            entity_name='active events')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def roll_group(self, group_id, group_roll):
        """
        Roll an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        group_roll (ElastigroupRoll): GroupRoll Object

        # Returns
        (Object): Elastigroup API response 
        """
        group_roll_request = aws_elastigroup.ElastigroupRollRequest(
            group_roll=group_roll)

        excluded_group_roll_dict = self.exclude_missing(
            json.loads(group_roll_request.toJSON()))

        formatted_group_roll_dict = self.convert_json(
            excluded_group_roll_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_roll_dict)

        roll_response = self.send_put(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/roll",
            body=body_json,
            entity_name='roll')

        formatted_response = self.convert_json(
            roll_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]

        return ret_val

    def get_all_group_deployment(self, group_id):
        """
        get all group deployment from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/roll",
            entity_name='roll')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def get_deployment_status(self, group_id, roll_id):
        """
        get all a deployment status from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        roll_id (String): Deployment ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/roll/" +
            str(roll_id) +
            "/status",
            entity_name='roll')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def stop_deployment(self, group_id, roll_id):
        """
        stop a deployment from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        roll_id (String): Deployment ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_put(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/roll/" +
            str(roll_id),
            body=json.dumps(dict(roll=dict(status="STOPPED"))),
            entity_name='roll')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]

    def create_deployment_action(self, group_id, roll_id, deployment_action):
        """
        create a deployment from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        roll_id (String): Deployment ID
        deployment_action (Deployment): Deployment Object

        # Returns
        (Object): Elastigroup API response 
        """
        deployment_action_request = aws_deployment_action.DeploymentActionRequest(
            deployment_action)

        deployment_action_dict = self.exclude_missing(
            json.loads(deployment_action_request.toJSON()))

        formatted_group = self.convert_json(
            deployment_action_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group)

        detach_response = self.send_post(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/roll/" +
            str(roll_id),
            body=body_json,
            entity_name='roll')

        formatted_response = self.convert_json(
            detach_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"]

        return ret_val

    def get_instance_healthiness(self, group_id):
        """
        Get a list of instances with health status.

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/instanceHealthiness",
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_cost_per_elastigroup(self, group_id, to_date=None, from_date=None):
        """
        get cost per elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        to_date (String) (Optional): Start Date
        from_date (String) (Optional): End Date

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(toDate=to_date, fromDate=from_date)

        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/costs",
            query_params=query_params,
            entity_name="cost"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_group_detailed_cost(self, group_id, to_date=None, from_date=None):
        """
        get detailed cost per elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        to_date (String) (Optional): Start Date
        from_date (String) (Optional): End Date

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(toDate=to_date, fromDate=from_date)

        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/costs/detailed",
            query_params=query_params,
            entity_name="cost"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def list_suspended_scaling_policies(self, group_id):
        """
        get suspended scaling policies for an elastigroup

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/scale/suspensions",
            entity_name="scaling policies"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def suspend_scaling_policies(self, group_id, policy_name):
        """
        suspended scaling policies for an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        policy_name (String): Scaling policy name

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(policyName=policy_name)

        response = self.send_post(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/scale/suspendPolicy",
            query_params=query_params,
            entity_name="scaling policies"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def resume_suspended_scaling_policies(self, group_id, policy_name):
        """
        Resume scaling policies for an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        policy_name (String): Scaling policy name

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(policyName=policy_name)

        response = self.send_post(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/scale/resumePolicy",
            query_params=query_params,
            entity_name="scaling policies"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def list_suspended_process(self, group_id):
        """
        List suspended process for an elastigroup

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/suspension",
            entity_name="suspend process"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def suspend_process(self, group_id, processes, suspensions):
        """
        suspended process for an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        processes (List): list of processes
        suspensions (List): list of suspensions

        # Returns
        (Object): Elastigroup API response 
        """
        if suspensions is not None:
            if processes is not None:
                # should fail in API
                item_to_send = dict(suspensions=suspensions,
                                    processes=processes)
            else:
                item_to_send = dict(suspensions=suspensions)
        else:
            item_to_send = dict(processes=processes)

        body = json.dumps(item_to_send)

        response = self.send_post(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/suspension",
            body=body,
            entity_name="suspend process"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def remove_suspended_process(self, group_id, processes):
        """
        remove suspended process for an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        processes (List): list of processes

        # Returns
        (Object): Elastigroup API response 
        """
        body = json.dumps(dict(processes=processes))

        response = self.send_delete_with_body(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/suspension",
            body=body,
            entity_name="suspend process"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response

    def detach_elastigroup_instances(self, group_id, detach_configuration):
        """
        Detatch instances from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        detatch_configuration (Detach): Detach Object

        # Returns
        (Object): Elastigroup API response 
        """
        group_detach_request = aws_elastigroup.ElastigroupDetachInstancesRequest(
            detach_configuration=detach_configuration)

        excluded_group_detach_dict = self.exclude_missing(
            json.loads(group_detach_request.toJSON()))

        formatted_group_detach_dict = self.convert_json(
            excluded_group_detach_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_detach_dict)

        detach_response = self.send_put(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/detachInstances",
            body=body_json,
            entity_name='detach')

        formatted_response = self.convert_json(
            detach_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["status"]

        return ret_val

    def deallocate_stateful_instance(self, group_id, stateful_instance_id):
        """
        Deallocate stateful instances from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        stateful_instance_id (String): Stateful Instance ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_put(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/statefulInstance/" +
            str(stateful_instance_id +
                "/deallocate"),
            entity_name='deallocate stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]

    def recycle_stateful_instance(self, group_id, stateful_instance_id):
        """
        Recycle stateful instances from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        stateful_instance_id (String): Stateful Instance ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_put(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/statefulInstance/" +
            str(stateful_instance_id +
                "/recycle"),
            entity_name='recycle stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]

    def get_stateful_instances(self, group_id):
        """
        Deallocate stateful instances from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        stateful_instance_id (String): Stateful Instance ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/statefulInstance",
            entity_name='get stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def resume_stateful_instance(self, group_id, stateful_instance_id):
        """
        Resume stateful instances from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        stateful_instance_id (String): Stateful Instance ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_put(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/statefulInstance/" +
            str(stateful_instance_id +
                "/resume"),
            entity_name='resume stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]

    def pause_stateful_instance(self, group_id, stateful_instance_id):
        """
        Pause stateful instances from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        stateful_instance_id (String): Stateful Instance ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_put(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/statefulInstance/" +
            str(stateful_instance_id) +
            "/pause",
            entity_name='pause stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]

    def delete_volume_in_stateful_instance(self, group_id, stateful_instance_id, volume_id):
        """
        Delete stateful instance 

        # Arguments
        group_id (String): Elastigroup ID
        stateful_instance_id (String): Stateful Instance ID
        volume_id (String): Volume ID

        # Returns
        (Object): Elastigroup API response 
        """
        return self.send_delete(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/statefulInstance/" +
            str(stateful_instance_id) +
            "/volume/" +
            str(volume_id),
            entity_name='delete volume in stateful instance')

    def beanstalk_maintenance_status(self, group_id):
        """
        Beanstalk maintenance status

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        status_response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/beanstalk/maintenance/status",
            entity_name="beanstalk maintenance start")

        formatted_response = self.convert_json(
            status_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"]

        return ret_val

    def beanstalk_maintenance_start(self, group_id):
        """
        Beanstalk maintenance start

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        start_response = self.send_put(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/beanstalk/maintenance/start",
            body={},
            entity_name="beanstalk maintenance start")

        formatted_response = self.convert_json(
            start_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["status"]

        return ret_val

    def beanstalk_maintenance_finish(self, group_id):
        """
        Beanstalk maintenance finish

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        finish_response = self.send_put(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/beanstalk/maintenance/finish",
            body={},
            entity_name="beanstalk maintenance start")

        formatted_response = self.convert_json(
            finish_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["status"]

        return ret_val

    def beanstalk_import(self, region, env_id=None, env_name=None):
        """
        Import beanstalk attributes into JSON. Either env_id or env_name is 
        required, both cannot be null

        # Arguments
        region (String): Beanstalk region
        env_id (String): Beanstalk env id
        env_name (String): Beanstalk env name

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(
            region=region, environmentId=env_id, environmentName=env_name)

        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/beanstalk/import",
            query_params=query_params,
            entity_name="beanstalk import"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def beanstalk_reimport(self, group_id):
        """
        Reimport beanstalk attributes

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_put(
            url=self.__base_elastigroup_url +
            "/" + str(group_id) +
            "/beanstalk/reimport",
            entity_name="beanstalk reimport"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def import_instance(self, region, instance_id, instance: aws_import.ImportInstanceConfig):
        """
        Import an EC2 instance into a new Elastigroup

        # Arguments
        region (String): Instance Region
        instance_id (String): Instance ID
        instance (ImportInstanceConfig): Import Configuration

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(
            region=region, instanceId=instance_id)

        request = aws_import.ImportInstanceRequest(instance)

        excluded_group_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_elastigroup_url +
            "/instance/import",
            query_params=query_params,
            entity_name='import instance')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def import_asg(self, region, asg_name, asg: aws_import.ASG, dry_run=None):
        """
        Create a new Elastigroup using the configuration of an existing Autoscaling group

        # Arguments
        region (String): ASG region
        asg_name (String): ASG Name
        asg (ASG): ASG Object
        dry_run (Bool) (Optional): if true only return JSON and not create group

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(
            region=region, autoScalingGroupName=asg_name, dryRun=dry_run)

        asg = aws_import.ImportASGRequest(asg)

        excluded_group_dict = self.exclude_missing(json.loads(asg.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_elastigroup_url +
            "/autoScalingGroup/import",
            query_params=query_params,
            entity_name='import asg')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def get_activity_events(self, group_id, from_date):
        """
        get activity events

        # Arguments
        group_id (String): Elastigroup ID
        from_date (String): From date

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(fromDate=from_date)

        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id + "/events",
            query_params=query_params,
            entity_name="activity groups"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"]

        return ret_val

    def ami_backup(self, group_id):
        """
        Start an AMI backup for Elastigroup

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_post(
            url=self.__base_elastigroup_url +
            "/" + group_id + "/amiBackup",
            entity_name="ami backup"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response

        return ret_val["response"]["status"]

    def create_blue_green_deployment(self, group_id, blue_green_deployment):
        """
        Start a Blue Green Deployment

        # Arguments
        group_id (String): Elastigroup ID
        blue_green_deployment (BGDeployment): Blue Green Deployment Object
        # Returns
        (Object): Elastigroup API response 
        """
        blue_green_deployment = aws_deployment.BlueGreenDeploymentRequest(
            blue_green_deployment)

        excluded_group_dict = self.exclude_missing(
            json.loads(blue_green_deployment.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_elastigroup_url + "/" +
            group_id + "/codeDeploy/blueGreenDeployment",
            entity_name='create b/g deployment')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def get_blue_green_deployment(self, group_id):
        """
        Get Blue Green Deployment for an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_get(
            url=self.__base_elastigroup_url + "/" +
            group_id + "/codeDeploy/blueGreenDeployment",
            entity_name="get b/g deployment")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def stop_blue_green_deployment(self, group_id, deployment_id):
        """
        Stop Blue Green Deployment for an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        deployment_id (String):  BG Deployment ID
        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_put(
            url=self.__base_elastigroup_url + "/" + group_id +
            "/codeDeploy/blueGreenDeployment/" + deployment_id + "/stop",
            entity_name="stop b/g deployment")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]

        return ret_val

    # endregion

    # region AWS
    def get_instance_type_by_region(self, region):
        """
        Get instance type by region

        # Arguments
        region (String): AWS region

        # Returns
        (Object): Spotinst API response 
        """
        query_params = dict(region=region)
        response = self.send_get(
            url=self.__base_aws_url +
            "/spotType",
            query_params=query_params,
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def lock_instance(self, instance_id, lock_time=None):
        """
        Lock instance 

        # Arguments
        instance_id (String): Instance ID
        lock_time (int) (Optinal): Time to lock instance

        # Returns
        (Object): Spotinst API response 
        """
        query_params = dict(ttlInMinutes=lock_time)

        response = self.send_post(
            url=self.__base_aws_url +
            "/instance/" +
            instance_id +
            "/lock",
            query_params=query_params,
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def unlock_instance(self, instance_id):
        """
        Unlock instance 

        # Arguments
        instance_id (String): Instance ID

        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_post(
            url=self.__base_aws_url +
            "/instance/" +
            instance_id +
            "/unlock",
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def enter_instance_standby(self, instance_id):
        """
        Enter standby for instance 

        # Arguments
        instance_id (String): Instance ID

        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_post(
            url=self.__base_aws_url +
            "/instance/" +
            instance_id +
            "/standby/enter",
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def exit_instance_standby(self, instance_id):
        """
        Exit standby for instance 

        # Arguments
        instance_id (String): Instance ID

        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_post(
            url=self.__base_aws_url +
            "/instance/" +
            instance_id +
            "/standby/exit",
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def get_instance_status(self, instance_id):
        """
        Get instance status

        # Arguments
        instance_id (String): Instance ID

        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url=self.__base_aws_url +
            "/instance/" +
            instance_id,
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def create_instance_signal(self, instance_id, signal):
        """
        create instance signal

        # Arguments
        instance_id (String): instance ID
        signal (String): Signal

        # Returns
        (Object): Elastigroup API response 
        """
        body = json.dumps(dict(instanceId=instance_id, signal=signal))

        response = self.send_post(
            url=self.__base_aws_url +
            "/instance/signal",
            body=body,
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def get_cost_per_account(self, to_date=None, from_date=None):
        """
        get cost per account

        # Arguments
        to_date (String) (Optional): to date
        from_date (String) (Optional): to date

        # Returns
        (Object): Spotinst API response 
        """
        query_params = dict(toDate=to_date, fromDate=from_date)

        response = self.send_get(
            url=self.__base_aws_url + "/costs",
            query_params=query_params,
            entity_name="cost"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_potential_savings(self):
        """
        get potential saving 

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_get(
            url=self.__base_aws_url + "/potentialSavings",
            entity_name="saving"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_instance_potential_savings(self, instance_ids, region):
        """
        get potential saving 

        # Arguments
        instance_ids (List): List of instance id strings
        region (String): region
        # Returns
        (Object): Elastigroup API response 
        """
        instance_str = ""

        for instance in instance_ids:
            instance_str += instance + ","

        query_params = dict(region=region, instanceIds=instance_str)

        response = self.send_get(
            url=self.__base_aws_url + "/instancePotentialSavings",
            query_params=query_params,
            entity_name="saving"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_elastilog(self, group_id, from_date, to_date, severity=None, resource_id=None, limit=None):
        """
        Get an elastilog for a specific elastigroup

        # Arguments
        group_id(String): Elastigroup ID
        to_date (String): to date
        from_date (String): to date
        severity (String) (Optional): Log level severity
        resource_id (String) (Optional): Filter log extracted entires related to a
          specific resource id
        limit(String) (Optional): Maximum number of lines to extract in a response

        # Returns
        (Object): Elastigroup API response 
        """
        geturl = self.__base_elastigroup_url + "/" + group_id + "/logs"
        query_params = dict(toDate=to_date, fromDate=from_date, severity=severity,
                            resource_id=resource_id, limit=limit)

        result = self.send_get(
            url=geturl, entity_name='elastilog', query_params=query_params)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    # endregion

    # region stateful
    def import_stateful_instance(self, stateful_instance):
        """
        Import stateful instance parametes

        # Arguments
        stateful_instance (StatefulInstance): StatefulInstance Object

        # Returns
        (Object): Elastigroup API response 
        """
        stateful_instance = aws_stateful.StatefulImportRequest(
            stateful_instance)

        excluded_group_dict = self.exclude_missing(
            json.loads(stateful_instance.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_stateful_url,
            entity_name='import stateful instance')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def get_stateful_import_status(self, stateful_migration_id):
        """
        Get stateful instance status

        # Arguments
        stateful_migration_id (String): Stateful migration ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_stateful_url +
            "/" +
            str(stateful_migration_id),
            entity_name='get stateful import status')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def delete_stateful_import(self, stateful_migration_id):
        """
        Delete stateful instance 

        # Arguments
        stateful_migration_id (String): Stateful migration ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_delete(
            url=self.__base_stateful_url +
            "/" +
            str(stateful_migration_id),
            entity_name='delete stateful import')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response

    # endregion


# endregion


# region GCP
class ElastigroupGcpClient(Client):
    __base_elastigroup_url = "/gcp/gce/group"
    __base_gcp_url = "/gcp/gce"

    def create_elastigroup(self, group: gcp_elastigroup.Elastigroup):
        """
        Create an elastigroup

        # Arguments
        group (Elastigroup): Elastigroup Object

        # Returns
        (Object): Elastigroup API response 
        """
        group = gcp_elastigroup.ElastigroupCreationRequest(group)

        excluded_group_dict = self.exclude_missing(json.loads(group.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_elastigroup_url,
            entity_name='elastigroup')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_elastigroup(self, group_update: gcp_elastigroup.Elastigroup, group_id: str):
        """
        Update an GCP Elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        group_update (Elastigroup): Elastigroup Object

        # Returns
        (Object): Elastigroup API response 
        """
        group = gcp_elastigroup.ElastigroupUpdateRequest(group_update)

        excluded_group_update_dict = self.exclude_missing(
            json.loads(group.toJSON()))

        formatted_group_update_dict = self.convert_json(
            excluded_group_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_update_dict)

        group_response = self.send_put(
            body=body_json,
            url=self.__base_elastigroup_url + "/" + group_id,
            entity_name='elastigroup'
        )

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_elastigroup(self, group_id: str):
        """
        Delete an Elastigroup GCP

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        delurl = self.__base_elastigroup_url + "/" + group_id
        response = self.send_delete(url=delurl, entity_name='elastigroup')
        return response

    def get_elastigroup(self, group_id: str):
        """
        List all properties for single GCP Elastigroup

        # Arguments
        group_id(String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        geturl = self.__base_elastigroup_url + "/" + group_id
        result = self.send_get(url=geturl, entity_name='elastigroup')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_elastigroups(self):
        """
        List all GCP Elastigroups for a Spot Account

        # Returns
        (List): List of Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_elastigroup_url,
            entity_name='elastigroup')
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    # region Scale
    def scale_elastigroup_up(self, group_id: str, adjustment: int):
        """
        Add instances to the Elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        adjustment (int): The number of instances to add to the group

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict({"adjustment": adjustment})
        content = self.send_put_with_params(
            url=self.__base_elastigroup_url + "/" + group_id + "/scale/up",
            entity_name='elastigroup (scale up)',
            body=None,
            user_query_params=query_params)

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def scale_elastigroup_down(self, group_id: str, adjustment: int):
        """
        Remove instances from the Elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        adjustment (int): Ammount to scale group

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict({"adjustment": adjustment})
        content = self.send_put_with_params(
            url=self.__base_elastigroup_url + "/" + group_id + "/scale/down",
            entity_name='elastigroup (scale down)',
            body=None,
            user_query_params=query_params)

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]
    # endregion

    # region Deploy
    def roll_group(self, group_id: str, group_roll: gcp_elastigroup.RollGroup):
        """
        Deploy the Elastigroup: Triggers a Blue/Green deployment that replaces the existing instances in the Elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        group_roll (ElastigroupRoll): GroupRoll Object

        # Returns
        (Object): Elastigroup API response 
        """
        group_roll_request = gcp_elastigroup.ElastigroupRollRequest(
            group_roll=group_roll)

        excluded_group_roll_dict = self.exclude_missing(
            json.loads(group_roll_request.toJSON()))

        formatted_group_roll_dict = self.convert_json(
            excluded_group_roll_dict, self.underscore_to_camel)

        roll_response = self.send_put(
            url=self.__base_elastigroup_url + "/" + group_id + "/roll",
            body=json.dumps(formatted_group_roll_dict),
            entity_name='roll')

        formatted_response = self.convert_json(
            roll_response, self.camel_to_underscore)

        return formatted_response["response"]

    def get_all_group_deployment(self, group_id: str):
        """
        Get all the deployments for a specific Elastigroup, and their status

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_elastigroup_url + "/" + group_id + "/roll",
            entity_name='roll')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def get_deployment_status(self, group_id: str, roll_id: str):
        """
        Get a specific deployment's status

        # Arguments
        group_id (String): The Spot Elastigroup ID you want to update
        roll_id (String): The deployment ID to query

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_elastigroup_url + "/" + group_id + "/roll/" + roll_id,
            entity_name='roll')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def stop_deployment(self, group_id: str, roll_id: str):
        """
        Stop an existing deployment

        # Arguments
        group_id (String): Elastigroup ID
        roll_id (String): Deployment ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_put(
            url=self.__base_elastigroup_url + "/" + group_id + "/roll/" + roll_id,
            body=json.dumps(dict(roll=dict(status="STOPPED"))),
            entity_name='roll')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]
    # endregion

    def get_elastigroup_active_instances(self, group_id: str):
        """
        Get the status for all instances that are memebers of the Elastigroup

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response 
        """
        content = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            group_id +
            "/status",
            entity_name='active instances')
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def get_cost_per_elastigroup(self, group_id: str, to_date: str, from_date: str):
        """
        Get financial information on a specific Elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        to_date (String) (Optional): Get items on or before this date (ISO 8601 or Unix timestamp)
        from_date (String) (Optional): Get items on or after this date (ISO 8601 or Unix timestamp)

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(toDate=to_date, fromDate=from_date)

        response = self.send_get(
            url=self.__base_elastigroup_url + "/" + group_id + "/costs",
            query_params=query_params,
            entity_name="cost"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_elastigroup_activity(self, group_id: str, start_date: str, end_date: str):
        """
        Get all activity events for the Elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        start_date (String): Date when to start checking
        end_date (String): Get items on or before this date (ISO 8601)

        # Returns
        (Object) : Elastigroup API response 
        """
        query_params = dict(fromDate=start_date, toDate=end_date)

        content = self.send_get(
            url=self.__base_elastigroup_url + "/" + group_id + "/events",
            query_params=query_params,
            entity_name='active events')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def detach_elastigroup_instances(self, group_id: str, detach_configuration: gcp_elastigroup.DetachConfiguration):
        """
        Detach instances from an Elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        detatch_configuration (DetachConfiguration): DetachConfiguration Object

        # Returns
        (Object): Elastigroup API response 
        """
        group_detach_request = gcp_elastigroup.ElastigroupDetachInstancesRequest(
            detach_configuration=detach_configuration)

        excluded_group_detach_dict = self.exclude_missing(
            json.loads(group_detach_request.toJSON()))

        formatted_group_detach_dict = self.convert_json(
            excluded_group_detach_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_detach_dict)

        detach_response = self.send_put(
            url=self.__base_elastigroup_url + "/" + group_id + "/detachInstances",
            body=body_json,
            entity_name='detach')

        formatted_response = self.convert_json(
            detach_response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def get_elastilog(self, group_id: str, start_date: str, end_date: str, limit: int):
        """
        Fetch a group's Elastilog

        # Arguments
        group_id (String): Elastigroup ID
        start_date (String): Get items on or after this date (ISO 8601 or Unix timestamp)
        end_date (String): Get items on or before this date (ISO 8601 or Unix timestamp)
        limit (Integer): Maximum number of items to return.

        # Returns
        (Object): Elastigroup API response
        """
        query_params = self.build_query_params_with_input(
            {"fromDate": start_date, "toDate": end_date, "limit": limit})

        content = self.send_get(
            url=self.__base_elastigroup_url + "/" + group_id + "/log",
            query_params=query_params,
            entity_name='active events')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def get_cost_per_account(self, start_date: str, end_date: str):
        """
        Retrieve costs up to one year back per specified account over a specified time period.

        # Arguments
        start_date (String): The start date of the requested time period. The value cannot be earlier than 1 year back.
        end_date (String): The end date of the requested time period. The maximum of time period length is 90 days.

        # Returns
        (Object): Cost per Account Response
        """
        query_params = dict(fromDate=start_date, toDate=end_date)

        content = self.send_get(
            url=self.__base_gcp_url + "/costs/",
            query_params=query_params,
            entity_name='account cost')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def get_instance_status(self, instance_id: str):
        """
        Get the current instance status. Possible status values: ACTIVE, TERMINATING

        # Arguments
        instance_id (String): GCP Instance ID

        # Returns
        (Object) Elastigroup API Response
        """
        content = self.send_get(
            url=self.__base_gcp_url + "/instance/" + instance_id,
            entity_name='instance status')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"][0]

    def lock_instance(self, instance_id: str, ttl_in_minutes: str):
        """
        Set termination protection for a specific instance.

        # Arguments
        instance_id (String): GCP Instance ID
        ttl_in_minutes (String): Specify a TTL (in minutes) for this lock, i.e.: for how long the protection will be valid for.

        # Returns
        (Object) Spotinst API Response
        """
        query_params = dict(ttlInMinutes=ttl_in_minutes)

        response = self.send_post_with_params(
            url=self.__base_gcp_url + "/instance/" + instance_id + "/lock",
            user_query_params=query_params,
            entity_name="lock instance",
            body=None
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def unlock_instance(self, instance_id: str):
        """
        Remove termination protection for a specific instance.

        # Arguments
        instance_id (String): GCP Instance ID

        # Returns
        (Object): Spotinst API response
        """
        response = self.send_post(
            url=self.__base_gcp_url + "/instance/" + instance_id + "/unlock",
            entity_name="unlock instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]
# endregion


# region Azure V3
class ElastigroupAzureV3Client(Client):
    __base_elastigroup_url = "/azure/compute/group"

    def create_elastigroup(self, group):
        """
        Create an elastigroup

        # Arguments
        group (Elastigroup): Elastigroup Object

        # Returns
        (Object): Elastigroup API response 
        """
        group = azure_v3_elastigroup.ElastigroupCreateRequest(group)

        excluded_group_dict = self.exclude_missing(json.loads(group.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_elastigroup_url,
            entity_name='elastigroup')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_elastigroup(self, group_update, group_id):
        """
        Update an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        group_update (Elastigroup): Elastigroup Object

        # Returns
        (Object): Elastigroup API response
        """
        group = azure_v3_elastigroup.ElastigroupUpdateRequest(group_update)

        excluded_group_update_dict = self.exclude_missing(
            json.loads(group.toJSON()))

        formatted_group_update_dict = self.convert_json(
            excluded_group_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_update_dict)

        group_response = self.send_put(
            body=body_json,
            url=self.__base_elastigroup_url + "/" + group_id,
            entity_name='elastigroup'
        )

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_elastigroup(self, group_id):
        """
        Delete an elastigroup

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response
        """
        delurl = self.__base_elastigroup_url + "/" + group_id
        return self.send_delete(url=delurl, entity_name='elastigroup')

    def get_elastigroup(self, group_id):
        """
        Get an elastigroup

        # Arguments
        group_id(String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response
        """
        geturl = self.__base_elastigroup_url + "/" + group_id
        result = self.send_get(url=geturl, entity_name='elastigroup')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_elastigroups(self):
        """
        Get all elastigroups

        # Returns
        (List): List of Elastigroup API response
        """
        content = self.send_get(
            url=self.__base_elastigroup_url, entity_name='elastigroup')
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def update_elastigroup_capacity(self, group_id, capacity):
        """
        Update capacity of Elastigroups

        # Arguments
        group_id (String): Elastigroup ID
        capacity: Capacity Object

        # Returns
        (Object): Elastigroup API response
        """
        update_capacity_request = azure_v3_elastigroup.ElastigroupUpdateCapacityRequest(
            capacity=capacity)

        excluded_update_capacity_dict = self.exclude_missing(
            json.loads(update_capacity_request.toJSON()))

        formatted_update_capacity_dict = self.convert_json(
            excluded_update_capacity_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_update_capacity_dict)

        response = self.send_put(url=self.__base_elastigroup_url + "/" +
                                 str(group_id) + "/capacity",
                                 body=body_json,
                                 entity_name='update capacity')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]

    def scale_elastigroup_up(self, group_id, adjustment):
        """
        Scale up an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        adjustment (int): Ammount to scale group

        # Returns
        (Object): Elastigroup API response
        """
        query_params = dict({"adjustment": adjustment})
        content = self.send_put_with_params(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/scale/up",
            entity_name='elastigroup (scale up)',
            body=None,
            user_query_params=query_params)

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]

    def scale_elastigroup_down(self, group_id, adjustment):
        """
        Scale down an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        adjustment (int): Ammount to scale group

        # Returns
        (Object): Elastigroup API response
        """
        query_params = dict({"adjustment": adjustment})
        content = self.send_put_with_params(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/scale/down",
            entity_name='elastigroup (scale down)',
            body=None,
            user_query_params=query_params)

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]

    def detach_elastigroup_vms(self, group_id, detach_configuration):
        """
        Detach VMs from an elastigroup

        # Arguments
        group_id (String): Elastigroup ID
        detach_configuration (Detach): DetachConfiguration Object

        # Returns
        (Object): Elastigroup API response
        """
        detach_request = azure_v3_elastigroup.ElastigroupDetachVMsRequest(
            detach_configuration=detach_configuration)

        excluded_detach_dict = self.exclude_missing(
            json.loads(detach_request.toJSON()))

        formatted_detach_dict = self.convert_json(
            excluded_detach_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_detach_dict)

        detach_response = self.send_put(
            url=self.__base_elastigroup_url +
            "/" + str(group_id) + "/detachVms",
            body=body_json,
            entity_name='detach')

        formatted_response = self.convert_json(
            detach_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]

        return ret_val

    def protect_virtual_machine(self, group_id, vm_name, ttl_in_minutes=None):
        """
        Protect virtual machines in Elastigroup cluster.

        # Arguments
        group_id (String): Elastigroup ID
        vm_name (String): VM ID
        ttl_in_minutes (int) (Optional): How long protection will be valid

        # Returns
        (Object): Spotinst API response
        """
        query_params = dict(ttlInMinutes=ttl_in_minutes)

        response = self.send_post(url=self.__base_elastigroup_url +
                                  "/" + str(group_id) +
                                  "/vm/" + str(vm_name) + "/protection",
                                  query_params=query_params,
                                  entity_name="virtual machine"
                                  )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def unprotect_virtual_machine(self, group_id, vm_name):
        """
        Un-Protect virtual machines in Elastigroup cluster.

        # Arguments
        group_id (String): Elastigroup ID
        vm_name (String): VM ID

        # Returns
        (Object): Spotinst API response
        """
        return self.send_delete(url=self.__base_elastigroup_url +
                                "/" + str(group_id) +
                                "/vm/" + str(vm_name) + "/protection",
                                entity_name="virtual machine")

    def get_elastigroup_status(self, group_id):
        """
        Get status of Elastigroup cluster.

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response
        """
        content = self.send_get(url=self.__base_elastigroup_url + "/" + group_id + "/status",
                                entity_name='elastigroup')
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"][0]

    def get_vm_healthiness(self, group_id):
        """
        Get a list of vms with health status.

        # Arguments
        group_id (String): Elastigroup ID

        # Returns
        (Object): Elastigroup API response
        """
        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/vmHealthiness",
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def suspend_elastigroup(self, group_id, processes):
        """
        Suspends the Group.

        # Arguments
        group_id (String): Elastigroup ID
        processes (List): List of processes to create or update their suspensions

        # Returns
        (Object): Spotinst API response
        """
        processes_request = azure_v3_elastigroup.ElastigroupProcessesRequest(
            processes=processes)

        excluded_process_dict = self.exclude_missing(
            json.loads(processes_request.toJSON()))

        formatted_process_dict = self.convert_json(
            excluded_process_dict, self.underscore_to_camel)

        body = json.dumps(formatted_process_dict)

        response = self.send_put(
            url=self.__base_elastigroup_url + "/" + group_id + "/suspend",
            body=body,
            entity_name='suspend')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def resume_elastigroup(self, group_id, processes):
        """
        Resumes the Group.

        # Arguments
        group_id (String): Elastigroup ID
        processes (List): List of processes to cancel their suspensions

        # Returns
        (Object): Spotinst API response
        """
        processes_request = azure_v3_elastigroup.ElastigroupProcessesRequest(
            processes=processes)

        excluded_process_dict = self.exclude_missing(
            json.loads(processes_request.toJSON()))

        formatted_process_dict = self.convert_json(
            excluded_process_dict, self.underscore_to_camel)

        body = json.dumps(formatted_process_dict)

        response = self.send_put(
            url=self.__base_elastigroup_url + "/" + group_id + "/resume",
            body=body,
            entity_name='resume')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def start_deployment(self, group_id, deployment):
        """
        Deploy the Elastigroup.
        This triggers a Blue/Green deployment that replaces the existing VMs in the Elastigroup.

        # Arguments
        group_id (String): Elastigroup ID
        deployment: DeploymentConfiguration Object

        # Returns
        (Object): Elastigroup API response
        """
        start_deployment_request = azure_v3_elastigroup.ElastigroupDeploymentRequest(
            deployment_configuration=deployment)

        excluded_start_deployment_dict = self.exclude_missing(
            json.loads(start_deployment_request.toJSON()))

        formatted_start_deployment_dict = self.convert_json(
            excluded_start_deployment_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_start_deployment_dict)

        response = self.send_post(url=self.__base_elastigroup_url + "/" +
                                  str(group_id) + "/deployment",
                                  body=body_json,
                                  entity_name='start deployment')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]

    def get_all_deployments(self, group_id, limit=None, sort=None):
        """
        Get a list of all the deployments of a specific Elastigroup and the status of each one.

        # Arguments
        group_id (String): Elastigroup ID
        limit (integer): Limits the number of deployments returned. Default: 5
        sort (String): Field by which to sort the results. Default: createdAt:DESC

        # Returns
        (Object) : Elastigroup API response
        """
        query_params = self.build_query_params_with_input(
            {"LIMIT": limit, "SORT": sort})

        content = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/deployment",
            query_params=query_params,
            entity_name='deployments')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def get_deployment(self, group_id, deployment_id):
        """
        Get the status of a specific deployment.

        # Arguments
        group_id (String): Elastigroup ID
        deployment_id (String): The deployment ID you want to query

        # Returns
        (Object) : Elastigroup API response
        """
        content = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/deployment/" +
            deployment_id,
            entity_name='deployment')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"][0]

    def get_deployment_status(self, group_id, deployment_id):
        """
        Get the detailed status of a specific deployment.
        This includes status details per batch and other information.

        # Arguments
        group_id (String): Elastigroup ID
        deployment_id (String): The deployment ID you want to query

        # Returns
        (Object) : Elastigroup API response
        """
        content = self.send_get(
            url=self.__base_elastigroup_url +
            "/" +
            str(group_id) +
            "/deployment/" +
            deployment_id +
            "/details",
            entity_name='deployment')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]

    def import_from_scale_set(self, resource_group_name, scale_set_name):
        """
        Given a scale set, constructs a valid group configuration based on the
          scale set and returns it.

        # Arguments
        resource_group_name (String): Resource Group Name
        scale_set_name (String): Scale Set Name

        # Returns
        (Object): Elastigroup API response
        """
        geturl = self.__base_elastigroup_url + "/import/resourceGroup/" + resource_group_name \
            + "/scaleSet/" + scale_set_name
        result = self.send_get(url=geturl, entity_name='elastigroup')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def import_from_virtual_machine(self, resource_group_name, virtual_machine_name):
        """
        Given a virtual machine, constructs a valid group configuration based on the 
        virtual machine and returns it.

        # Arguments
        resource_group_name (String): Resource Group Name
        virtual_machine_name (String): Virtual Machine Name

        # Returns
        (Object): Elastigroup API response
        """
        geturl = self.__base_elastigroup_url + "/import/resourceGroup/" + resource_group_name \
            + "/virtualMachine/" + virtual_machine_name
        result = self.send_get(url=geturl, entity_name='elastigroup')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def import_from_load_balancer(self, backend_pool_name, load_balancer_name, resource_group_name):
        """
        Given a load balancer, constructs a valid group configuration and returns it.

        # Arguments
        resource_group_name (String): Resource Group Name
        load_balancer_name (String): Virtual Machine Name
        backend_pool_name (String): Backend Pool Name

        # Returns
        (Object): Elastigroup API response
        """
        geturl = self.__base_elastigroup_url + "/import/resourceGroup/" + resource_group_name \
            + "/loadBalancer/" + load_balancer_name + "/backendPool/" + backend_pool_name
        result = self.send_get(url=geturl, entity_name='elastigroup')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def import_from_application_gateway(self, backend_pool_name, application_gateway_name,
                                        resource_group_name):
        """
        Given a load balancer, constructs a valid group configuration and returns it.

        # Arguments
        resource_group_name (String): Resource Group Name
        application_gateway_name (String): Application Gateway Name
        backend_pool_name (String): Backend Pool Name

        # Returns
        (Object): Elastigroup API response
        """
        geturl = self.__base_elastigroup_url + "/import/resourceGroup/" + resource_group_name \
            + "/applicationGateway/" + application_gateway_name + \
            "/backendPool/" + backend_pool_name
        result = self.send_get(url=geturl, entity_name='elastigroup')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def create_vm_signal(self, vm_name, signal_type):
        """
        The VM signal API is used for notifying Spot about the VM state so that
          Spot can act accordingly

        # Arguments
        vm_name (String): The virtual machine ID the signal refers to.
        signal_type (String): Signal Type (Enum: "vmReady" "vmReadyToShutdown")

        # Returns
        (Object): Elastigroup API response
        """
        body = json.dumps(dict(vmName=vm_name, signalType=signal_type))

        response = self.send_post(
            url="/azure/compute/vm/signal",
            body=body,
            entity_name="vm signal"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def get_elastilog(self, group_id, from_date, to_date, severity=None,
                      resource_id=None, limit=None):
        """
        Get an elastilog for a specific elastigroup

        # Arguments
        group_id(String): Elastigroup ID
        to_date (String): to date
        from_date (String): from date
        severity(String) (Optional): Log level severity
        resource_id(String) (Optional): Filter log extracted entires 
        related to a specific resource id
        limit(String) (Optional): Maximum number of lines to extract in a response

        # Returns
        (Object): Elastigroup API response
        """
        geturl = self.__base_elastigroup_url + "/" + group_id + "/logs"
        query_params = dict(toDate=to_date, fromDate=from_date, SEVERITY=severity,
                            RESOURCE_ID=resource_id, limit=limit)

        result = self.send_get(
            url=geturl, entity_name='elastilog', query_params=query_params)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"]

        # endregion



import json

from spotinst_sdk2.client import Client

# region Azure imports
import spotinst_sdk2.models.setup.azure as azure_setup
import spotinst_sdk2.models.setup.gcp as gcp_setup
# endregion

# region AWS


class SetupAWSClient(Client):
    __base_setup_url = "/setup"

    def create_external_id(self):
        """
        Create aws account external id.
        You should use the external id when creating your AWS role for your spot account

        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_post(
            url=self.__base_setup_url + "/credentials/aws/externalId",
            entity_name="credentials"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val

    def set_credentials(self, iam_role):
        """
        Set aws credentials 
        Please create external id using spot api (see #AdminClient.create_aws_external_id)
        and use it when creating the AWS role

        # Arguments
        iam_role (String): IAM Role

        # Returns
        (Object): Spotinst API response 
        """
        credentials = {"iamRole": iam_role}

        response = self.send_post(
            url=self.__base_setup_url +
            "/credentials/aws",
            body=json.dumps(dict(credentials=credentials)),
            entity_name="credentials"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["status"]

        return ret_val
# endregion


# region Azure
class SetupAzureClient(Client):
    __base_setup_url = "/azure/setup"

    def set_credentials(self, credentials):
        """"
        Set azure credentials 

        # Arguments
        credentials (AzureCredentials): Azure Credentials Object

        # Returns
        (Object): Spotinst API response 
        """
        set_credentials_request = azure_setup.AzureSetCredentialsRequest(
            azure_credentials=credentials)

        excluded_set_credentials_dict = self.exclude_missing(
            json.loads(set_credentials_request.to_json()))

        formatted_set_credentials_dict = self.convert_json(
            excluded_set_credentials_dict, self.underscore_to_camel)

        response = self.send_post(
            url=self.__base_setup_url + "/credentials",
            body=json.dumps(formatted_set_credentials_dict),
            entity_name='credentials')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["status"]

        return ret_val

    def validate_credentials(self, credentials):
        """"
        Validate azure credentials 

        # Arguments
        credentials (AzureCredentials): Azure Credentials Object

        # Returns
        (Object): Spotinst API response 
        """
        set_credentials_request = azure_setup.AzureSetCredentialsRequest(
            azure_credentials=credentials)

        excluded_set_credentials_dict = self.exclude_missing(
            json.loads(set_credentials_request.to_json()))

        formatted_set_credentials_dict = self.convert_json(
            excluded_set_credentials_dict, self.underscore_to_camel)

        response = self.send_post(
            url=self.__base_setup_url + "/credentials/validation",
            body=json.dumps(formatted_set_credentials_dict),
            entity_name='credentials')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["status"]

        return ret_val
# endregion


# region GCP
class SetupGCPClient(Client):
    __base_setup_url = "/gcp/setup"

    def set_credentials(self, credentials):
        """
        Set gcp credentials 

        # Arguments
        credentials (GcpCredentials): Gcp Credentials Object

        # Returns
        (Object): Spotinst API response 
        """
        set_credentials_request = gcp_setup.GcpSetCredentialsRequest(
            gcp_credentials=credentials)

        excluded_set_credentials_dict = self.exclude_missing(
            json.loads(set_credentials_request.to_json()))

        response = self.send_post(
            url=self.__base_setup_url + "/credentials",
            body=json.dumps(excluded_set_credentials_dict),
            entity_name='credentials')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["status"]

        return ret_val

    def validate_credentials(self, credentials):
        """
        Validate gcp credentials 

        # Arguments
        credentials (GcpCredentials): Gcp Credentials Object

        # Returns
        (Object): Spotinst API response 
        """
        set_credentials_request = gcp_setup.GcpSetCredentialsRequest(
            gcp_credentials=credentials)

        excluded_set_credentials_dict = self.exclude_missing(
            json.loads(set_credentials_request.to_json()))

        response = self.send_post(
            url=self.__base_setup_url + "/credentials/validation",
            body=json.dumps(excluded_set_credentials_dict),
            entity_name='credentials')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["status"]

        return ret_val
# endregion



import json
from enum import Enum
from typing import List

none = "d3043820717d74d9a17694c176d39733"


# region SpotDeployment
class SpotDeployment:
    """
    # Arguments
    cluster_id: str
    name: str
    namespace: str
    """

    def __init__(
            self,
            cluster_id: str = none,
            name: str = none,
            namespace: str = none):
        self.cluster_id = cluster_id
        self.name = name
        self.namespace = namespace
# endregion


# region Strategy
class SecretRef:
    """
    # Arguments
    name: str
    key: str
    """

    def __init__(
            self,
            name: str = none,
            key: str = none):
        self.name = name
        self.key = key


class FieldRef:
    """
    # Arguments
    field_path: str
    """

    def __init__(
            self,
            field_path: str = none):
        self.field_path = field_path


class ValueFrom:
    """
    # Arguments
    secret_ref: SecretRef
    field_ref: FieldRef
    """

    def __init__(
            self,
            secret_ref: SecretRef = none,
            field_ref: FieldRef = none):
        self.secret_ref = secret_ref
        self.field_ref = field_ref


class Argument:
    """
    # Arguments
    name: str
    value: str
    value_from: ValueFrom
    """

    def __init__(
            self,
            name: str = none,
            value: str = none,
            value_from: ValueFrom = none):
        self.name = name
        self.value = value
        self.value_from = value_from


class Strategy:
    """
    # Arguments
    name: str
    args: List[Argument]
    """

    def __init__(
            self,
            name: str = none,
            args: List[Argument] = none):
        self.name = name
        self.args = args
# endregion


# region Traffic
class TlsRoutes:
    """
    # Arguments
    port: int
    sni_hosts: List[str]
    """

    def __init__(
            self,
            port: int = none,
            sni_hosts: List[str] = none):
        self.port = port
        self.sni_hosts = sni_hosts


class VirtualServices:
    """
    # Arguments
    name: str
    routes: List[str]
    tls_routes: List[TlsRoutes]
    """

    def __init__(
            self,
            name: str = none,
            routes: List[str] = none,
            tls_routes: List[TlsRoutes] = none):
        self.name = name
        self.routes = routes
        self.tls_routes = tls_routes


class DestinationRule:
    """
    # Arguments
    name: str
    canary_subset_name: str
    stable_subset_name: str
    """

    def __init__(
            self,
            name: str = none,
            canary_subset_name: str = none,
            stable_subset_name: str = none):
        self.name = name
        self.canary_subset_name = canary_subset_name
        self.stable_subset_name = stable_subset_name


class Istio:
    """
    # Arguments
    virtual_services: List[VirtualServices]
    destination_rule: DestinationRule
    """

    def __init__(
            self,
            virtual_services: List[VirtualServices] = none,
            destination_rule: DestinationRule = none):
        self.virtual_services = virtual_services
        self.destination_rule = destination_rule


class Nginx:
    """
    # Arguments
    annotation_prefix: str
    stable_ingress: str
    """

    def __init__(
            self,
            annotation_prefix: str = none,
            stable_ingress: str = none):
        self.annotation_prefix = annotation_prefix
        self.stable_ingress = stable_ingress


class StickinessConfig:
    """
    # Arguments
    enabled: bool
    duration_seconds: int
    """

    def __init__(
            self,
            enabled: bool = none,
            duration_seconds: int = none):
        self.enabled = enabled
        self.duration_seconds = duration_seconds


class Alb:
    """
    # Arguments
    ingress: str
    service_port: int
    root_service: str
    stickiness_config: StickinessConfig
    annotation_prefix: str
    """

    def __init__(
            self,
            ingress: str = none,
            service_port: int = none,
            root_service: str = none,
            stickiness_config: StickinessConfig = none,
            annotation_prefix: str = none):
        self.ingress = ingress
        self.service_port = service_port
        self.root_service = root_service
        self.stickiness_config = stickiness_config
        self.annotation_prefix = annotation_prefix


class Smi:
    """
    # Arguments
    root_service: str
    traffic_split_name: str
    """

    def __init__(
            self,
            root_service: str = none,
            traffic_split_name: str = none):
        self.root_service = root_service
        self.traffic_split_name = traffic_split_name


class Ambassador:
    """
    # Arguments
    mappings: List[str]
    """

    def __init__(
            self,
            mappings: List[str] = none):
        self.mappings = mappings


class VirtualService:
    """
    # Arguments
    name: str
    routes: List[str]
    """

    def __init__(
            self,
            name: str = none,
            routes: List[str] = none):
        self.name = name
        self.routes = routes


class CanaryVirtualNodeRef:
    """
    # Arguments
    name: str
    """

    def __init__(
            self,
            name: str = none):
        self.name = name


class StableVirtualNodeRef:
    """
    # Arguments
    name: str
    """

    def __init__(
            self,
            name: str = none):
        self.name = name


class VirtualNodeGroup:
    """
    # Arguments
    canary_virtual_node_ref: CanaryVirtualNodeRef
    stable_virtual_node_ref: StableVirtualNodeRef
    """

    def __init__(
            self,
            canary_virtual_node_ref: CanaryVirtualNodeRef = none,
            stable_virtual_node_ref: StableVirtualNodeRef = none):
        self.canary_virtual_node_ref = canary_virtual_node_ref
        self.stable_virtual_node_ref = stable_virtual_node_ref


class AppMesh:
    """
    # Arguments
    virtual_service: VirtualService
    virtual_node_group: VirtualNodeGroup
    """

    def __init__(
            self,
            virtual_service: VirtualService = none,
            virtual_node_group: VirtualNodeGroup = none):
        self.virtual_service = virtual_service
        self.virtual_node_group = virtual_node_group


class PingPong:
    """
    ping_service: str
    pong_service: str
    """

    def __init__(
            self,
            ping_service: str = none,
            pong_service: str = none):
        self.ping_service = ping_service
        self.pong_service = pong_service


class Traffic:
    """
    # Arguments
    canary_service: str
    stable_service: str
    istio: Istio
    nginx: Nginx
    alb: Alb
    smi: Smi
    ambassador: Ambassador
    app_mesh: AppMesh
    ping_pong: PingPong
    """

    def __init__(
            self,
            canary_service: str = none,
            stable_service: str = none,
            istio: Istio = none,
            nginx: Nginx = none,
            alb: Alb = none,
            smi: Smi = none,
            ambassador: Ambassador = none,
            app_mesh: AppMesh = none,
            ping_pong: PingPong = none):
        self.canary_service = canary_service
        self.stable_service = stable_service
        self.istio = istio
        self.nginx = nginx
        self.alb = alb
        self.smi = smi
        self.ambassador = ambassador
        self.app_mesh = app_mesh
        self.ping_pong = ping_pong
# endregion


# region Failure Policy
class FailureAction(Enum):
    abort = "abort"
    pause = "pause"
    promote = "promote"


class FailurePolicy:
    """
    # Arguments
    action: FailureAction
    """

    def __init__(
            self,
            action: FailureAction = none):
        self.action = action
# endregion


# region rolloutSpec
class RolloutSpec:
    """
    # Arguments
    name: str
    spot_deployment: SpotDeployment
    strategy: Strategy
    traffic: Traffic
    failure_policy: FailurePolicy
    """

    def __init__(
            self,
            name: str = none,
            spot_deployment: SpotDeployment = none,
            strategy: Strategy = none,
            traffic: Traffic = none,
            failure_policy: FailurePolicy = none):
        self.name = name
        self.spot_deployment = spot_deployment
        self.strategy = strategy
        self.traffic = traffic
        self.failure_policy = failure_policy
# endregion


# region Client Requests
class CreateRolloutSpecRequest:
    """
    # Arguments
    rollout_spec : RolloutSpec
    """

    def __init__(self, rollout_spec: RolloutSpec):
        self.rollout_spec = rollout_spec

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion



import json

from spotinst_sdk2.client import Client

import spotinst_sdk2.models.hpc.aws as aws_hpc


class HPCAwsClient(Client):
    __base_hpc_url = "/hpc/aws/lsf/cluster"

    def create_hpc_cluster(self, cluster: aws_hpc.HPC):
        """
        Create an hpc cluster

        # Arguments
        group (HPCCluster): HPCCluster Object

        # Returns
        (Object): HPCCluster API response 
        """
        cluster = aws_hpc.LSFClusterCreationRequest(cluster)

        excluded_cluster_dict = self.exclude_missing(
            json.loads(cluster.toJSON()))

        formatted_cluster_dict = self.convert_json(
            excluded_cluster_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_cluster_dict)

        cluster_response = self.send_post(
            body=body_json,
            url=self.__base_hpc_url,
            entity_name='hpc_cluster')

        formatted_response = self.convert_json(
            cluster_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_hpc_cluster(self, cluster_id: str):
        """
        Delete a hpc cluster

        # Arguments
        group_id (String): Cluster ID

        # Returns
        (Object): HPC Cluster API response 
        """
        delurl = self.__base_hpc_url + "/" + cluster_id
        return self.send_delete(url=delurl, entity_name='hpc_cluster')

    def get_hpc_cluster(self, cluster_id: str):
        """
        Get an hpc cluster

        # Arguments
        group_id(String): Cluster ID

        # Returns
        (Object): HPC Cluster API response 
        """
        geturl = self.__base_hpc_url + "/" + cluster_id
        result = self.send_get(url=geturl, entity_name='hpc_cluster')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_hpc_clusters(self):
        """
        Get all hpc clusters
        # Returns
        (List): List of HPC Cluster API response
        """
        content = self.send_get(
            url=self.__base_hpc_url,
            entity_name='hpc_cluster')
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def update_hpc_cluster(self, hpc_cluster_update: aws_hpc.HPC, cluster_id: str):
        """
        Update hpc cluster
        # Arguments
        cluster_id (String): Cluster ID
        hpc_cluster_update (HPC Cluster): HPC Cluster Object

        # Returns
        (Object): HPC Cluster API response
        """
        cluster = aws_hpc.LSFClusterUpdateRequest(hpc_cluster_update)

        excluded_cluster_update_dict = self.exclude_missing(
            json.loads(cluster.toJSON()))

        formatted_cluster_update_dict = self.convert_json(
            excluded_cluster_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_cluster_update_dict)

        cluster_response = self.send_put(
            body=body_json,
            url=self.__base_hpc_url + "/" + cluster_id,
            entity_name='hpc_cluster'
        )

        formatted_response = self.convert_json(
            cluster_response, self.camel_to_underscore)

        ret_val = formatted_response["response"]["items"][0]

        return ret_val
